@@ -1,7 +1,6 @@
 /****************************************
 *  Computer Algebra System SINGULAR     *
 ****************************************/
-
 /*
 * ABSTRACT: interpreter:
 *           assignment of expressions and lists to objects or lists
@@ -11,90 +10,105 @@
 #include <string.h>
 #include <ctype.h>
 
+
 #include <kernel/mod2.h>
-#include <Singular/tok.h>
-#include <kernel/options.h>
-#include <Singular/ipid.h>
-#include <kernel/idrec.h>
-#include <kernel/intvec.h>
-#include <kernel/bigintmat.h>
+
 #include <omalloc/omalloc.h>
-#include <kernel/febase.h>
+
+#define TRANSEXT_PRIVATES
+#include <polys/ext_fields/transext.h>
+
+#include <misc/options.h>
+#include <misc/intvec.h>
+
+#include <coeffs/coeffs.h>
+#include <coeffs/numbers.h>
+#include <coeffs/bigintmat.h>
+
+
+#include <polys/ext_fields/algext.h>
+
+#include <polys/monomials/ring.h>
+#include <polys/matpol.h>
+#include <polys/monomials/maps.h>
+#include <polys/nc/nc.h>
+#include <polys/nc/sca.h>
+#include <polys/prCopy.h>
+
 #include <kernel/polys.h>
 #include <kernel/ideals.h>
-#include <kernel/matpol.h>
-#include <kernel/kstd1.h>
-#include <kernel/timer.h>
-#include <kernel/ring.h>
-#include <Singular/subexpr.h>
-#include <Singular/lists.h>
-#include <kernel/numbers.h>
-#include <kernel/longalg.h>
-#include <kernel/stairc.h>
-#include <kernel/maps.h>
-#include <kernel/syz.h>
+#include <kernel/GBEngine/kstd1.h>
+#include <kernel/oswrapper/timer.h>
+#include <kernel/combinatorics/stairc.h>
+#include <kernel/GBEngine/syz.h>
+
 //#include "weight.h"
-#include <Singular/ipconv.h>
-#include <Singular/attrib.h>
-#include <Singular/silink.h>
-#include <Singular/ipshell.h>
-#include <kernel/sca.h>
-#include <Singular/blackbox.h>
+#include "tok.h"
+#include "ipid.h"
+#include "idrec.h"
+#include "subexpr.h"
+#include "lists.h"
+#include "ipconv.h"
+#include "attrib.h"
+#include "links/silink.h"
+#include "ipshell.h"
+#include "blackbox.h"
+#include "Singular/number2.h"
 
 /*=================== proc =================*/
-static BOOLEAN jjECHO(leftv res, leftv a)
+static BOOLEAN jjECHO(leftv, leftv a)
 {
   si_echo=(int)((long)(a->Data()));
   return FALSE;
 }
-static BOOLEAN jjPRINTLEVEL(leftv res, leftv a)
+static BOOLEAN jjPRINTLEVEL(leftv, leftv a)
 {
   printlevel=(int)((long)(a->Data()));
   return FALSE;
 }
-static BOOLEAN jjCOLMAX(leftv res, leftv a)
+static BOOLEAN jjCOLMAX(leftv, leftv a)
 {
   colmax=(int)((long)(a->Data()));
   return FALSE;
 }
-static BOOLEAN jjTIMER(leftv res, leftv a)
+static BOOLEAN jjTIMER(leftv, leftv a)
 {
   timerv=(int)((long)(a->Data()));
   initTimer();
   return FALSE;
 }
 #ifdef HAVE_GETTIMEOFDAY
-static BOOLEAN jjRTIMER(leftv res, leftv a)
+static BOOLEAN jjRTIMER(leftv, leftv a)
 {
   rtimerv=(int)((long)(a->Data()));
   initRTimer();
   return FALSE;
 }
 #endif
-static BOOLEAN jjMAXDEG(leftv res, leftv a)
+static BOOLEAN jjMAXDEG(leftv, leftv a)
 {
   Kstd1_deg=(int)((long)(a->Data()));
   if (Kstd1_deg!=0)
-    test |=Sy_bit(OPT_DEGBOUND);
+    si_opt_1 |=Sy_bit(OPT_DEGBOUND);
   else
-    test &=(~Sy_bit(OPT_DEGBOUND));
+    si_opt_1 &=(~Sy_bit(OPT_DEGBOUND));
   return FALSE;
 }
-static BOOLEAN jjMAXMULT(leftv res, leftv a)
+static BOOLEAN jjMAXMULT(leftv, leftv a)
 {
   Kstd1_mu=(int)((long)(a->Data()));
   if (Kstd1_mu!=0)
-    test |=Sy_bit(OPT_MULTBOUND);
+    si_opt_1 |=Sy_bit(OPT_MULTBOUND);
   else
-    test &=(~Sy_bit(OPT_MULTBOUND));
+    si_opt_1 &=(~Sy_bit(OPT_MULTBOUND));
   return FALSE;
 }
-static BOOLEAN jjTRACE(leftv res, leftv a)
+static BOOLEAN jjTRACE(leftv, leftv a)
 {
   traceit=(int)((long)(a->Data()));
   return FALSE;
 }
-static BOOLEAN jjSHORTOUT(leftv res, leftv a)
+static BOOLEAN jjSHORTOUT(leftv, leftv a)
 {
   if (currRing != NULL)
   {
@@ -109,6 +123,12 @@ static BOOLEAN jjSHORTOUT(leftv res, leftv a)
     }
 #else
     currRing->ShortOut = shortOut;
+    coeffs cf = currRing->cf;
+    while (nCoeff_is_Extension(cf)) {
+      cf->extRing->ShortOut = shortOut;
+      assume(cf->extRing != NULL);
+      cf = cf->extRing->cf;
+    }
 #endif
   }
   return FALSE;
@@ -130,7 +150,7 @@ static void jjMINPOLY_red(idhdl h)
     case POLY_CMD:
     {
       poly p=(poly)IDDATA(h);
-      IDDATA(h)=(char*)pMinPolyNormalize(p);
+      IDDATA(h)=(char*)p_MinPolyNormalize(p, currRing);
       break;
     }
     case IDEAL_CMD:
@@ -140,7 +160,8 @@ static void jjMINPOLY_red(idhdl h)
     {
       int i;
       ideal I=(ideal)IDDATA(h);
-      for(i=IDELEMS(I)-1;i>=0;i--) I->m[i]=pMinPolyNormalize(I->m[i]);
+      for(i=IDELEMS(I)-1;i>=0;i--)
+             I->m[i]=p_MinPolyNormalize(I->m[i], currRing);
       break;
     }
     case LIST_CMD:
@@ -157,55 +178,144 @@ static void jjMINPOLY_red(idhdl h)
        Werror("type %d too complex...set minpoly before",IDTYP(h)); break;
   }
 }
-static BOOLEAN jjMINPOLY(leftv res, leftv a)
+static BOOLEAN jjMINPOLY(leftv, leftv a)
 {
-  number p=(number)a->CopyD(NUMBER_CMD);
-  if (nIsZero(p))
+  if( !nCoeff_is_transExt(currRing->cf) && (currRing->idroot == NULL) && n_IsZero((number)a->Data(), currRing->cf) )
   {
-    currRing->minpoly=NULL;
-    naMinimalPoly=NULL;
+#ifndef SING_NDEBUG
+    WarnS("Set minpoly over non-transcendental ground field to 0?!");
+    Warn("in >>%s<<",my_yylinebuf);
+#endif
+    return FALSE;
+  }
+
+
+  if ( !nCoeff_is_transExt(currRing->cf) )
+  {
+    WarnS("Trying to set minpoly over non-transcendental ground field...");
+    if(!nCoeff_is_algExt(currRing->cf) )
+    {
+      WerrorS("cannot set minpoly for these coeffients");
+      return TRUE;
+    }
+  }
+  if ((rVar(currRing->cf->extRing)!=1)
+  && !n_IsZero((number)a->Data(), currRing->cf) )
+  {
+    WerrorS("only univarite minpoly allowed");
+    return TRUE;
+  }
+
+  if ( currRing->idroot != NULL )
+  {
+//    return TRUE;
+#ifndef SING_NDEBUG
+    idhdl p = currRing->idroot;
+
+    WarnS("no minpoly allowed if there are local objects belonging to the basering: ");
+
+    while(p != NULL)
+    {
+      PrintS(p->String(TRUE)); PrintLn();
+      p = p->next;
+    }
+#endif
+  }
+
+//  assume (currRing->idroot==NULL);
+
+  number p = (number)a->CopyD(NUMBER_CMD);
+  n_Normalize(p, currRing->cf);
+
+  if (n_IsZero(p, currRing->cf))
+  {
+    n_Delete(&p, currRing->cf);
+    if( nCoeff_is_transExt(currRing->cf) )
+    {
+#ifndef SING_NDEBUG
+      WarnS("minpoly is already 0...");
+#endif
+      return FALSE;
+    }
+    WarnS("cannot set minpoly to 0 / alg. extension?");
+    return TRUE;
+  }
+
+  // remove all object currently in the ring
+  while(currRing->idroot!=NULL)
+  {
+#ifndef SING_NDEBUG
+    Warn("killing a local object due to minpoly change: %s", IDID(currRing->idroot));
+#endif
+    killhdl2(currRing->idroot,&(currRing->idroot),currRing);
+  }
+
+  AlgExtInfo A;
+
+  A.r = rCopy(currRing->cf->extRing); // Copy  ground field!
+  // if minpoly was already set:
+  if( currRing->cf->extRing->qideal != NULL ) id_Delete(&(A.r->qideal),A.r);
+  ideal q = idInit(1,1);
+  if ((p==NULL) ||(NUM((fraction)p)==NULL))
+  {
+    WerrorS("Could not construct the alg. extension: minpoly==0");
+    // cleanup A: TODO
+    rDelete( A.r );
+    return TRUE;
+  }
+  if (DEN((fraction)(p)) != NULL) // minpoly must be a fraction with poly numerator...!!
+  {
+    poly z=NUM((fraction)p);
+    poly n=DEN((fraction)(p));
+    z=p_Mult_nn(z,pGetCoeff(n),currRing->cf->extRing);
+    NUM((fraction)p)=z;
+    DEN((fraction)(p))=NULL;
+    p_Delete(&n,currRing->cf->extRing);
+  }
+
+  q->m[0] = NUM((fraction)p);
+  A.r->qideal = q;
+
+#if 0
+  PrintS("\nTrying to conver the currRing into an algebraic field: ");
+  PrintS("Ground poly. ring: \n");
+  rWrite( A.r );
+  PrintS("\nGiven MinPOLY: ");
+  p_Write( A.i->m[0], A.r );
+#endif
+
+  // :(
+//  NUM((fractionObject *)p) = NULL; // makes 0/ NULL fraction - which should not happen!
+//  n_Delete(&p, currRing->cf); // doesn't expect 0/ NULL :(
+  if(true)
+  {
+    extern omBin fractionObjectBin;
+    NUM((fractionObject *)p) = NULL; // not necessary, but still...
+    omFreeBin((ADDRESS)p, fractionObjectBin);
+  }
+
+
+  coeffs new_cf = nInitChar(n_algExt, &A);
+
+  if (new_cf==NULL)
+  {
+    WerrorS("Could not construct the alg. extension: llegal minpoly?");
+    // cleanup A: TODO
+    rDelete( A.r );
+    return TRUE;
   }
   else
   {
-    if ((rPar(currRing)!=1)
-      || (rField_is_GF()))
-    {
-      WerrorS("no minpoly allowed");
-      return TRUE;
-    }
-    if (currRing->minpoly!=NULL)
-    {
-      WerrorS("minpoly already set");
-      return TRUE;
-    }
-    nNormalize(p);
-    currRing->minpoly=p;
-    naMinimalPoly=((lnumber)currRing->minpoly)->z;
-    if (p_GetExp(((lnumber)currRing->minpoly)->z,1,currRing->algring)==0)
-    {
-      Werror("minpoly must not be constant");
-      currRing->minpoly=NULL;
-      naMinimalPoly=NULL;
-      nDelete(&p);
-    }
-    /* redefine function pointers due to switch from
-       transcendental to algebraic field extension */
-    //redefineFunctionPointers();
-    // and now, normalize all already defined objects in this ring
-    idhdl h=currRing->idroot;
-    while(h!=NULL)
-    {
-      jjMINPOLY_red(h);
-      h=IDNEXT(h);
-    }
+    nKillChar(currRing->cf); currRing->cf=new_cf;
   }
+
   return FALSE;
 }
-static BOOLEAN jjNOETHER(leftv res, leftv a)
+static BOOLEAN jjNOETHER(leftv, leftv a)
 {
   poly p=(poly)a->CopyD(POLY_CMD);
-  pDelete(&ppNoether);
-  ppNoether=p;
+  pDelete(&(currRing->ppNoether));
+  (currRing->ppNoether)=p;
   return FALSE;
 }
 /*=================== proc =================*/
@@ -277,7 +387,7 @@ static BOOLEAN jiA_INT(leftv res, leftv a, Subexpr e)
       int c=e->next->start;
       if ((i>=iv->rows())||(c<1)||(c>iv->cols()))
       {
-        Werror("wrong range [%d,%d] in intmat (%d,%d)",i+1,c,iv->rows(),iv->cols());
+        Werror("wrong range [%d,%d] in intmat %s(%d,%d)",i+1,c,res->Name(),iv->rows(),iv->cols());
         return TRUE;
       }
       else
@@ -286,7 +396,7 @@ static BOOLEAN jiA_INT(leftv res, leftv a, Subexpr e)
   }
   return FALSE;
 }
-static BOOLEAN jiA_NUMBER(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_NUMBER(leftv res, leftv a, Subexpr)
 {
   number p=(number)a->CopyD(NUMBER_CMD);
   if (res->data!=NULL) nDelete((number *)&res->data);
@@ -295,12 +405,187 @@ static BOOLEAN jiA_NUMBER(leftv res, leftv a, Subexpr e)
   jiAssignAttr(res,a);
   return FALSE;
 }
+#ifdef SINGULAR_4_2
+static BOOLEAN jiA_NUMBER2(leftv res, leftv a, Subexpr e)
+{
+  number2 n=(number2)a->CopyD(CNUMBER_CMD);
+  if (e==NULL)
+  {
+    if (res->data!=NULL)
+    {
+      number2 nn=(number2)res->data;
+      n2Delete(nn);
+    }
+    res->data=(void *)n;
+    jiAssignAttr(res,a);
+  }
+  else
+  {
+    int i=e->start-1;
+    if (i<0)
+    {
+      Werror("index[%d] must be positive",i+1);
+      return TRUE;
+    }
+    bigintmat *iv=(bigintmat *)res->data;
+    if (e->next==NULL)
+    {
+      WerrorS("only one index given");
+      return TRUE;
+    }
+    else
+    {
+      int c=e->next->start;
+      if ((i>=iv->rows())||(c<1)||(c>iv->cols()))
+      {
+        Werror("wrong range [%d,%d] in cmatrix %s(%d,%d)",i+1,c,res->Name(),iv->rows(),iv->cols());
+        return TRUE;
+      }
+      else if (iv->basecoeffs()==n->cf)
+      {
+        n_Delete((number *)&BIMATELEM(*iv,i+1,c),iv->basecoeffs());
+        BIMATELEM(*iv,i+1,c) = n->n;
+      }
+      else
+      {
+        WerrorS("different base");
+        return TRUE;
+      }
+    }
+  }
+  jiAssignAttr(res,a);
+  return FALSE;
+}
+static BOOLEAN jiA_NUMBER2_I(leftv res, leftv a, Subexpr e)
+{
+  if (e==NULL)
+  {
+    if (res->data!=NULL)
+    {
+      number2 nn=(number2)res->data;
+      number2 n=n2Init((long)a->Data(),nn->cf);
+      n2Delete(nn);
+      res->data=(void *)n;
+    }
+    else
+    {
+      WerrorS("no Ring avialable for conversion from int");
+      return TRUE;
+    }
+  }
+  else
+  {
+    int i=e->start-1;
+    if (i<0)
+    {
+      Werror("index[%d] must be positive",i+1);
+      return TRUE;
+    }
+    bigintmat *iv=(bigintmat *)res->data;
+    if (e->next==NULL)
+    {
+      WerrorS("only one index given");
+      return TRUE;
+    }
+    else
+    {
+      int c=e->next->start;
+      if ((i>=iv->rows())||(c<1)||(c>iv->cols()))
+      {
+        Werror("wrong range [%d,%d] in cmatrix %s(%d,%d)",i+1,c,res->Name(),iv->rows(),iv->cols());
+        return TRUE;
+      }
+      else
+      {
+        n_Delete((number *)&BIMATELEM(*iv,i+1,c),iv->basecoeffs());
+        BIMATELEM(*iv,i+1,c) = n_Init((long)a->Data(),iv->basecoeffs());
+      }
+    }
+  }
+  return FALSE;
+}
+static BOOLEAN jiA_NUMBER2_N(leftv res, leftv a, Subexpr e)
+{
+  if (e==NULL)
+  {
+    if (res->data!=NULL)
+    {
+      number2 nn=(number2)res->data;
+      number2 n=(number2)omAlloc(sizeof(*n));
+      n->cf=currRing->cf; n->cf->ref++;
+      n->n=(number)a->CopyD(NUMBER_CMD);
+      n2Delete(nn);
+      res->data=(void *)n;
+    }
+    else
+    {
+      number2 n=(number2)omAlloc(sizeof(*n));
+      n->cf=currRing->cf; n->cf->ref++;
+      n->n=(number)a->CopyD(NUMBER_CMD);
+      res->data=(void *)n;
+    }
+  }
+  else return TRUE; // TODO: list elements
+  return FALSE;
+}
+static BOOLEAN jiA_POLY2(leftv res, leftv a, Subexpr e)
+{
+  poly2 n=(poly2)a->CopyD(CPOLY_CMD);
+  if (e==NULL)
+  {
+    if (res->data!=NULL)
+    {
+      poly2 nn=(poly2)res->data;
+      p2Delete(nn);
+    }
+    res->data=(void *)n;
+    jiAssignAttr(res,a);
+  }
+  else
+  {
+    int i=e->start-1;
+    if (i<0)
+    {
+      Werror("index[%d] must be positive",i+1);
+      return TRUE;
+    }
+    WerrorS("not yet"); // TODO: list elem
+    return TRUE;
+  }
+  jiAssignAttr(res,a);
+  return FALSE;
+}
+static BOOLEAN jiA_POLY2_P(leftv res, leftv a, Subexpr e)
+{
+  if (e==NULL)
+  {
+    if (res->data!=NULL)
+    {
+      poly2 nn=(poly2)res->data;
+      poly2 n=(poly2)omAlloc(sizeof(*n));
+      n->cf=currRing; n->cf->ref++;
+      n->n=(poly)a->CopyD(POLY_CMD);
+      p2Delete(nn);
+      res->data=(void *)n;
+    }
+    else
+    {
+      poly2 n=(poly2)omAlloc(sizeof(*n));
+      n->cf=currRing; n->cf->ref++;
+      n->n=(poly)a->CopyD(POLY_CMD);
+      res->data=(void *)n;
+    }
+  }
+  else return TRUE; // TODO: list elements
+  return FALSE;
+}
+#endif
 static BOOLEAN jiA_BIGINT(leftv res, leftv a, Subexpr e)
 {
   number p=(number)a->CopyD(BIGINT_CMD);
   if (e==NULL)
   {
-    if (res->data!=NULL) nlDelete((number *)&res->data,NULL);
+    if (res->data!=NULL) n_Delete((number *)&res->data,coeffs_BIGINT);
     res->data=(void *)p;
   }
   else
@@ -322,12 +607,12 @@ static BOOLEAN jiA_BIGINT(leftv res, leftv a, Subexpr e)
       int c=e->next->start;
       if ((i>=iv->rows())||(c<1)||(c>iv->cols()))
       {
-        Werror("wrong range [%d,%d] in bigintmat (%d,%d)",i+1,c,iv->rows(),iv->cols());
+        Werror("wrong range [%d,%d] in bigintmat %s(%d,%d)",i+1,c,res->Name(),iv->rows(),iv->cols());
         return TRUE;
       }
       else
       {
-        nlDelete((number *)&BIMATELEM(*iv,i+1,c),NULL);
+        n_Delete((number *)&BIMATELEM(*iv,i+1,c),iv->basecoeffs());
         BIMATELEM(*iv,i+1,c) = p;
       }
     }
@@ -335,7 +620,7 @@ static BOOLEAN jiA_BIGINT(leftv res, leftv a, Subexpr e)
   jiAssignAttr(res,a);
   return FALSE;
 }
-static BOOLEAN jiA_LIST_RES(leftv res, leftv a,Subexpr e)
+static BOOLEAN jiA_LIST_RES(leftv res, leftv a,Subexpr)
 {
   syStrategy r=(syStrategy)a->CopyD(RESOLUTION_CMD);
   if (res->data!=NULL) ((lists)res->data)->Clean();
@@ -346,7 +631,7 @@ static BOOLEAN jiA_LIST_RES(leftv res, leftv a,Subexpr e)
   //jiAssignAttr(res,a);
   return FALSE;
 }
-static BOOLEAN jiA_LIST(leftv res, leftv a,Subexpr e)
+static BOOLEAN jiA_LIST(leftv res, leftv a,Subexpr)
 {
   lists l=(lists)a->CopyD(LIST_CMD);
   if (res->data!=NULL) ((lists)res->data)->Clean();
@@ -360,10 +645,15 @@ static BOOLEAN jiA_POLY(leftv res, leftv a,Subexpr e)
   pNormalize(p);
   if (e==NULL)
   {
+    if ((p!=NULL) && TEST_V_QRING && (currRing->qideal!=NULL)
+    && (!hasFlag(a,FLAG_QRING)))
+    {
+      jjNormalizeQRingP(p);
+      setFlag(res,FLAG_QRING);
+    }
     if (res->data!=NULL) pDelete((poly*)&res->data);
     res->data=(void*)p;
     jiAssignAttr(res,a);
-    if (TEST_V_QRING && (currQuotient!=NULL) && (!hasFlag(res,FLAG_QRING))) jjNormalizeQRingP(res);
   }
   else
   {
@@ -376,6 +666,10 @@ static BOOLEAN jiA_POLY(leftv res, leftv a,Subexpr e)
       // for all ideal like data types: check indices
       if (j>MATCOLS(m))
       {
+        if (TEST_V_ALLWARN)
+        {
+          Warn("increase ideal %d -> %d in %s",MATCOLS(m),j,my_yylinebuf);
+        }
         pEnlargeSet(&(m->m),MATCOLS(m),j-MATCOLS(m));
         MATCOLS(m)=j;
       }
@@ -390,6 +684,10 @@ static BOOLEAN jiA_POLY(leftv res, leftv a,Subexpr e)
       // for matrices: indices are correct (see ipExprArith3(..,'['..) )
       j=e->next->start;
     }
+    if ((p!=NULL) && TEST_V_QRING && (currRing->qideal!=NULL))
+    {
+      jjNormalizeQRingP(p);
+    }
     pDelete(&MATELEM(m,i,j));
     MATELEM(m,i,j)=p;
     /* for module: update rank */
@@ -397,13 +695,12 @@ static BOOLEAN jiA_POLY(leftv res, leftv a,Subexpr e)
     {
       m->rank=si_max(m->rank,pMaxComp(p));
     }
-    if (TEST_V_QRING) jjNormalizeQRingP(res);
   }
   return FALSE;
 }
 static BOOLEAN jiA_1x1INTMAT(leftv res, leftv a,Subexpr e)
 {
-  if ((res->rtyp!=INTMAT_CMD) /*|| (e!=NULL) - TRUE because of type int */)
+  if (/*(*/ res->rtyp!=INTMAT_CMD /*)*/) /*|| (e!=NULL) - TRUE because of type int */
   {
     // no error message: assignment simply fails
     return TRUE;
@@ -425,7 +722,7 @@ static BOOLEAN jiA_1x1INTMAT(leftv res, leftv a,Subexpr e)
 }
 static BOOLEAN jiA_1x1MATRIX(leftv res, leftv a,Subexpr e)
 {
-  if ((res->rtyp!=MATRIX_CMD) /*|| (e!=NULL) - TRUE because of type poly */)
+  if (/*(*/ res->rtyp!=MATRIX_CMD /*)*/) /*|| (e!=NULL) - TRUE because of type poly */
   {
     // no error message: assignment simply fails
     return TRUE;
@@ -470,15 +767,13 @@ static BOOLEAN jiA_STRING(leftv res, leftv a, Subexpr e)
   }
   return FALSE;
 }
-static BOOLEAN jiA_PROC(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_PROC(leftv res, leftv a, Subexpr)
 {
   extern procinfo *iiInitSingularProcinfo(procinfo *pi, const char *libname,
                                           const char *procname, int line,
                                           long pos, BOOLEAN pstatic=FALSE);
-  extern void piCleanUp(procinfov pi);
-
-  if(res->data!=NULL) piCleanUp((procinfo *)res->data);
-  if(a->rtyp==STRING_CMD)
+  if(res->data!=NULL) piKill((procinfo *)res->data);
+  if(a->Typ()==STRING_CMD)
   {
     res->data = (void *)omAlloc0Bin(procinfo_bin);
     ((procinfo *)(res->data))->language=LANG_NONE;
@@ -486,11 +781,11 @@ static BOOLEAN jiA_PROC(leftv res, leftv a, Subexpr e)
     ((procinfo *)res->data)->data.s.body=(char *)a->CopyD(STRING_CMD);
   }
   else
-    res->data=(void *)a->CopyD();
+    res->data=(void *)a->CopyD(PROC_CMD);
   jiAssignAttr(res,a);
   return FALSE;
 }
-static BOOLEAN jiA_INTVEC(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_INTVEC(leftv res, leftv a, Subexpr)
 {
   //if ((res->data==NULL) || (res->Typ()==a->Typ()))
   {
@@ -513,19 +808,19 @@ static BOOLEAN jiA_INTVEC(leftv res, leftv a, Subexpr e)
   }
 #endif
 }
-static BOOLEAN jiA_BIGINTMAT(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_BIGINTMAT(leftv res, leftv a, Subexpr)
 {
   if (res->data!=NULL) delete ((bigintmat *)res->data);
   res->data=(void *)a->CopyD(BIGINTMAT_CMD);
   jiAssignAttr(res,a);
   return FALSE;
 }
-static BOOLEAN jiA_IDEAL(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_IDEAL(leftv res, leftv a, Subexpr)
 {
   if (res->data!=NULL) idDelete((ideal*)&res->data);
   res->data=(void *)a->CopyD(MATRIX_CMD);
-  if (a->rtyp==IDHDL) idNormalize((ideal)a->Data());
-  else                idNormalize((ideal)res->data);
+  if (a->rtyp==IDHDL) id_Normalize((ideal)a->Data(), currRing);
+  else                id_Normalize((ideal)res->data, currRing);
   jiAssignAttr(res,a);
   if (((res->rtyp==IDEAL_CMD)||(res->rtyp==MODUL_CMD))
   && (IDELEMS((ideal)(res->data))==1)
@@ -535,17 +830,18 @@ static BOOLEAN jiA_IDEAL(leftv res, leftv a, Subexpr e)
   {
     setFlag(res,FLAG_STD);
   }
-  if (TEST_V_QRING && (currQuotient!=NULL)&& (!hasFlag(res,FLAG_QRING))) jjNormalizeQRingId(res);
+  if (TEST_V_QRING && (currRing->qideal!=NULL)&& (!hasFlag(res,FLAG_QRING))) jjNormalizeQRingId(res);
   return FALSE;
 }
-static BOOLEAN jiA_RESOLUTION(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_RESOLUTION(leftv res, leftv a, Subexpr)
 {
   if (res->data!=NULL) syKillComputation((syStrategy)res->data);
   res->data=(void *)a->CopyD(RESOLUTION_CMD);
   jiAssignAttr(res,a);
   return FALSE;
 }
-static BOOLEAN jiA_MODUL_P(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_MODUL_P(leftv res, leftv a, Subexpr)
+/* module = poly */
 {
   if (res->data!=NULL) idDelete((ideal*)&res->data);
   ideal I=idInit(1,1);
@@ -553,26 +849,29 @@ static BOOLEAN jiA_MODUL_P(leftv res, leftv a, Subexpr e)
   if (I->m[0]!=NULL) pSetCompP(I->m[0],1);
   pNormalize(I->m[0]);
   res->data=(void *)I;
-  if (TEST_V_QRING && (currQuotient!=NULL))
+  if (TEST_V_QRING && (currRing->qideal!=NULL))
   {
     if (hasFlag(a,FLAG_QRING)) setFlag(res,FLAG_QRING);
     else                       jjNormalizeQRingId(res);
   }
   return FALSE;
 }
-static BOOLEAN jiA_IDEAL_M(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_IDEAL_M(leftv res, leftv a, Subexpr)
 {
   if (res->data!=NULL) idDelete((ideal*)&res->data);
   matrix m=(matrix)a->CopyD(MATRIX_CMD);
+  if (TEST_V_ALLWARN)
+    if (MATROWS(m)>1)
+      Warn("assign matrix with %d rows to an ideal in >>%s<<",MATROWS(m),my_yylinebuf);
   IDELEMS((ideal)m)=MATROWS(m)*MATCOLS(m);
   ((ideal)m)->rank=1;
   MATROWS(m)=1;
-  idNormalize((ideal)m);
+  id_Normalize((ideal)m, currRing);
   res->data=(void *)m;
-  if (TEST_V_QRING && (currQuotient!=NULL)) jjNormalizeQRingId(res);
+  if (TEST_V_QRING && (currRing->qideal!=NULL)) jjNormalizeQRingId(res);
   return FALSE;
 }
-static BOOLEAN jiA_LINK(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_LINK(leftv res, leftv a, Subexpr)
 {
   si_link l=(si_link)res->data;
 
@@ -596,7 +895,7 @@ static BOOLEAN jiA_LINK(leftv res, leftv a, Subexpr e)
   return TRUE;
 }
 // assign map -> map
-static BOOLEAN jiA_MAP(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_MAP(leftv res, leftv a, Subexpr)
 {
   if (res->data!=NULL)
   {
@@ -609,7 +908,7 @@ static BOOLEAN jiA_MAP(leftv res, leftv a, Subexpr e)
   return FALSE;
 }
 // assign ideal -> map
-static BOOLEAN jiA_MAP_ID(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_MAP_ID(leftv res, leftv a, Subexpr)
 {
   map f=(map)res->data;
   char *rn=f->preimage; // save the old/already assigned preimage ring name
@@ -617,7 +916,7 @@ static BOOLEAN jiA_MAP_ID(leftv res, leftv a, Subexpr e)
   idDelete((ideal *)&f);
   res->data=(void *)a->CopyD(IDEAL_CMD);
   f=(map)res->data;
-  idNormalize((ideal)f);
+  id_Normalize((ideal)f, currRing);
   f->preimage = rn;
   return FALSE;
 }
@@ -630,49 +929,73 @@ static BOOLEAN jiA_QRING(leftv res, leftv a,Subexpr e)
     WerrorS("qring_id expected");
     return TRUE;
   }
+  ring old_ring=(ring)res->Data();
 
-  ring qr;
+  coeffs newcf = currRing->cf;
+  ideal id = (ideal)a->Data(); //?
+  const int cpos = idPosConstant(id);
+  if(rField_is_Ring(currRing))
+    if (cpos >= 0)
+    {
+        newcf = n_CoeffRingQuot1(p_GetCoeff(id->m[cpos], currRing), currRing->cf);
+        if(newcf == NULL)
+          return TRUE;
+    }
   //qr=(ring)res->Data();
   //if (qr!=NULL) omFreeBin((ADDRESS)qr, ip_sring_bin);
-  assume(res->Data()==NULL);
-  qr=rCopy(currRing);
+  ring qr = rCopy(currRing);
+  assume(qr->cf == currRing->cf);
+
+  if ( qr->cf != newcf )
+  {
+    nKillChar ( qr->cf ); // ???
+    qr->cf = newcf;
+  }
                  // we have to fill it, but the copy also allocates space
   idhdl h=(idhdl)res->data; // we have res->rtyp==IDHDL
   IDRING(h)=qr;
 
-  ideal id=(ideal)a->CopyD(IDEAL_CMD);
+  ideal qid;
 
-  if ((idElem(id)>1) || rIsSCA(currRing) || (currRing->qideal!=NULL))
-    assumeStdFlag(a);
-
-#ifdef HAVE_RINGS
-  if (rField_is_Ring(currRing))
+  if((rField_is_Ring(currRing)) && (cpos != -1))
   {
-    if (idPosConstant(id) != -1)
-    {
-      WerrorS("constant in q-ideal; please modify ground field/ring instead");
-      return TRUE;
-    }
+    int i, j;
+    int *perm = (int *)omAlloc0((qr->N+1)*sizeof(int));
+
+    for(i=qr->N;i>0;i--)
+      perm[i]=i;
+
+    nMapFunc nMap = n_SetMap(currRing->cf, newcf);
+    qid = idInit(IDELEMS(id)-1,1);
+    for(i = 0, j = 0; i<IDELEMS(id); i++)
+      if( i != cpos )
+        qid->m[j++] = p_PermPoly(id->m[i], perm, currRing, qr, nMap, NULL, 0);
   }
-#endif
+  else
+    qid = idrCopyR(id,currRing,qr);
+
+  idSkipZeroes(qid);
+  //idPrint(qid);
+  if ((idElem(qid)>1) || rIsSCA(currRing) || (currRing->qideal!=NULL))
+    assumeStdFlag(a);
 
   if (currRing->qideal!=NULL) /* we are already in a qring! */
   {
-    ideal tmp=idSimpleAdd(id,currRing->qideal);
+    ideal tmp=idSimpleAdd(qid,currRing->qideal);
     // both ideals should be GB, so dSimpleAdd is sufficient
-    idDelete(&id);
-    id=tmp;
+    idDelete(&qid);
+    qid=tmp;
     // delete the qr copy of quotient ideal!!!
     idDelete(&qr->qideal);
   }
-  if (idElem(id)==0)
+  if (idElem(qid)==0)
   {
     qr->qideal = NULL;
-    idDelete(&id);
+    id_Delete(&qid,currRing);
     IDTYP(h)=RING_CMD;
   }
   else
-    qr->qideal = id;
+    qr->qideal = qid;
 
   // qr is a copy of currRing with the new qideal!
   #ifdef HAVE_PLURAL
@@ -689,7 +1012,12 @@ static BOOLEAN jiA_QRING(leftv res, leftv a,Subexpr e)
     }
   }
   #endif
+  //rWrite(qr);
   rSetHdl((idhdl)res->data);
+  if (old_ring!=NULL)
+  {
+    rDelete(old_ring);
+  }
   return FALSE;
 }
 
@@ -703,6 +1031,7 @@ static BOOLEAN jiA_RING(leftv res, leftv a, Subexpr e)
     have_id=FALSE;
   }
   ring r=(ring)a->Data();
+  if ((r==NULL)||(r->cf==NULL)) return TRUE;
   if (have_id)
   {
     idhdl rl=(idhdl)res->data;
@@ -724,27 +1053,36 @@ static BOOLEAN jiA_RING(leftv res, leftv a, Subexpr e)
   jiAssignAttr(res,a);
   return FALSE;
 }
-static BOOLEAN jiA_PACKAGE(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_PACKAGE(leftv res, leftv a, Subexpr)
 {
   res->data=(void *)a->CopyD(PACKAGE_CMD);
   jiAssignAttr(res,a);
   return FALSE;
 }
-static BOOLEAN jiA_DEF(leftv res, leftv a, Subexpr e)
+static BOOLEAN jiA_DEF(leftv res, leftv, Subexpr)
 {
   res->data=(void *)0;
   return FALSE;
 }
+#ifdef SINGULAR_4_1
+static BOOLEAN jiA_CRING(leftv res, leftv a, Subexpr e)
+{
+  res->data=(void *)a->CopyD(CRING_CMD);
+  jiAssignAttr(res,a);
+  return FALSE;
+}
+#endif
+
 /*=================== table =================*/
 #define IPASSIGN
 #define D(A)     A
 #define NULL_VAL NULL
-#include <Singular/table.h>
+#include "table.h"
 /*=================== operations ============================*/
 /*2
 * assign a = b
 */
-static BOOLEAN jiAssign_1(leftv l, leftv r)
+static BOOLEAN jiAssign_1(leftv l, leftv r, BOOLEAN toplevel)
 {
   int rt=r->Typ();
   if (rt==0)
@@ -754,7 +1092,7 @@ static BOOLEAN jiAssign_1(leftv l, leftv r)
   }
 
   int lt=l->Typ();
-  if((lt==0)/*&&(l->name!=NULL)*/)
+  if (lt==0)
   {
     if (!errorreported) Werror("left side `%s` is undefined",l->Fullname());
     return TRUE;
@@ -762,25 +1100,24 @@ static BOOLEAN jiAssign_1(leftv l, leftv r)
   if(rt==NONE)
   {
     WarnS("right side is not a datum, assignment ignored");
+    Warn("in line >>%s<<",my_yylinebuf);
     // if (!errorreported)
     //   WerrorS("right side is not a datum");
     //return TRUE;
     return FALSE;
   }
 
-  int i=0;
-
   if (lt==DEF_CMD)
   {
     if (TEST_V_ALLWARN
     && (rt!=RING_CMD)
-    && (rt!=QRING_CMD)
     && (l->name!=NULL)
     && (l->e==NULL)
     && (iiCurrArgs==NULL) /* not in proc header */
     )
     {
-      Warn("use `%s` instead of `def`",Tok2Cmdname(rt));
+      Warn("use `%s` instead of `def` in %s:%d:%s",Tok2Cmdname(rt),
+            currentVoice->filename,yylineno,my_yylinebuf);
     }
     if (l->rtyp==IDHDL)
     {
@@ -804,8 +1141,16 @@ static BOOLEAN jiAssign_1(leftv l, leftv r)
       return FALSE;
   }
   leftv ld=l;
-  if ((l->rtyp==IDHDL)&&(lt!=QRING_CMD)&&(lt!=RING_CMD))
-    ld=(leftv)l->data;
+  if (l->rtyp==IDHDL)
+  {
+    if (lt!=RING_CMD)
+      ld=(leftv)l->data;
+  }
+  else if (toplevel)
+  {
+    WerrorS("error in assign: left side is not an l-value");
+    return TRUE;
+  }
   if (lt>MAX_TOK)
   {
     blackbox *bb=getBlackboxStuff(lt);
@@ -814,10 +1159,13 @@ static BOOLEAN jiAssign_1(leftv l, leftv r)
 #endif
     return (bb==NULL) || bb->blackbox_Assign(l,r);
   }
-  while (((dAssign[i].res!=lt)
-      || (dAssign[i].arg!=rt))
-    && (dAssign[i].res!=0)) i++;
-  if (dAssign[i].res!=0)
+  int start=0;
+  while ((dAssign[start].res!=lt)
+      && (dAssign[start].res!=0)) start++;
+  int i=start;
+  while ((dAssign[i].res==lt)
+      && (dAssign[i].arg!=rt)) i++;
+  if (dAssign[i].res==lt)
   {
     if (traceit&TRACE_ASSIGN) Print("assign %s=%s\n",Tok2Cmdname(lt),Tok2Cmdname(rt));
     BOOLEAN b;
@@ -830,14 +1178,14 @@ static BOOLEAN jiAssign_1(leftv l, leftv r)
     return b;
   }
   // implicite type conversion ----------------------------------------------
-  if (dAssign[i].res==0)
+  if (dAssign[i].res!=lt)
   {
     int ri;
     leftv rn = (leftv)omAlloc0Bin(sleftv_bin);
     BOOLEAN failed=FALSE;
-    i=0;
-    while ((dAssign[i].res!=lt)
-      && (dAssign[i].res!=0)) i++;
+    i=start;
+    //while ((dAssign[i].res!=lt)
+    //  && (dAssign[i].res!=0)) i++;
     while (dAssign[i].res==lt)
     {
       if ((ri=iiTestConvert(rt,dAssign[i].arg))!=0)
@@ -985,10 +1333,10 @@ static BOOLEAN jiA_INTVEC_L(leftv l,leftv r)
   t.rtyp=INT_CMD;
   while ((i<iv->length())&&(l!=NULL))
   {
-    t.data=(char *)(*iv)[i];
+    t.data=(char *)(long)(*iv)[i];
     h=l->next;
     l->next=NULL;
-    nok=jiAssign_1(l,&t);
+    nok=jiAssign_1(l,&t,TRUE);
     l->next=h;
     if (nok) return TRUE;
     i++;
@@ -1022,7 +1370,7 @@ static BOOLEAN jiA_VECTOR_L(leftv l,leftv r)
     }
     h=l->next;
     l->next=NULL;
-    nok=jiAssign_1(l,&t);
+    nok=jiAssign_1(l,&t,TRUE);
     l->next=h;
     t.CleanUp();
     if (nok)
@@ -1036,7 +1384,7 @@ static BOOLEAN jiA_VECTOR_L(leftv l,leftv r)
   idDelete(&I);
   l1->CleanUp();
   r->CleanUp();
-  //if (TEST_V_QRING && (currQuotient!=NULL)) jjNormalizeQRingP(l);
+  //if (TEST_V_QRING && (currRing->qideal!=NULL)) jjNormalizeQRingP(l);
   return FALSE;
 }
 static BOOLEAN jjA_L_LIST(leftv l, leftv r)
@@ -1071,7 +1419,7 @@ static BOOLEAN jjA_L_LIST(leftv l, leftv r)
       //listall();
       goto err;
     }
-    //if ((rt==RING_CMD)||(rt==QRING_CMD))
+    //if (rt==RING_CMD)
     //{
     //  L->m[i].rtyp=rt;
     //  L->m[i].data=h->Data();
@@ -1092,7 +1440,7 @@ static BOOLEAN jjA_L_LIST(leftv l, leftv r)
   {
     IDLIST((idhdl)l->data)=L;
     IDTYP((idhdl)l->data)=LIST_CMD; // was possibly DEF_CMD
-    ipMoveId((idhdl)l->data);
+    if (lRingDependend(L)) ipMoveId((idhdl)l->data);
   }
   else
   {
@@ -1116,7 +1464,7 @@ static BOOLEAN jjA_L_INTVEC(leftv l,leftv r,intvec *iv)
       if (traceit&TRACE_ASSIGN)
       {
         Warn("expression list length(%d) does not match intmat size(%d)",
-              iv->length()+exprlist_length(hh),iv->length());
+             iv->length()+exprlist_length(hh),iv->length());
       }
       break;
     }
@@ -1161,20 +1509,20 @@ static BOOLEAN jjA_L_BIGINTMAT(leftv l,leftv r,bigintmat *bim)
   if (bim->length()==0) { WerrorS("bigintmat is 1x0"); delete bim; return TRUE; }
   while (hh!=NULL)
   {
-    if (i>=bim->length())
+    if (i>=bim->cols()*bim->rows())
     {
       if (traceit&TRACE_ASSIGN)
       {
-        Warn("expression list length(%d) does not match bigintmat size(%d)",
-              bim->length()+exprlist_length(hh),bim->length());
+        Warn("expression list length(%d) does not match bigintmat size(%d x %d)",
+              exprlist_length(hh),bim->rows(),bim->cols());
       }
       break;
     }
     if (hh->Typ() == INT_CMD)
     {
-      number tp = nlInit((int)((long)(hh->Data())), NULL);
+      number tp = n_Init((int)((long)(hh->Data())), coeffs_BIGINT);
       bim->set(i++, tp);
-      nlDelete(&tp, NULL);
+      n_Delete(&tp, coeffs_BIGINT);
     }
     else if (hh->Typ() == BIGINT_CMD)
     {
@@ -1260,7 +1608,7 @@ static BOOLEAN jiA_MATRIX_L(leftv l,leftv r)
       l->next=NULL;
       idhdl hh=NULL;
       if ((l->rtyp==IDHDL)&&(l->Typ()==DEF_CMD)) hh=(idhdl)l->data;
-      nok=jiAssign_1(l,&t);
+      nok=jiAssign_1(l,&t,TRUE);
       if (hh!=NULL) { ipMoveId(hh);hh=NULL;}
       l->next=h;
       if (nok)
@@ -1336,7 +1684,7 @@ static BOOLEAN jiA_STRING_L(leftv l,leftv r)
     t.data=ss;
     h=l->next;
     l->next=NULL;
-    nok=jiAssign_1(l,&t);
+    nok=jiAssign_1(l,&t,TRUE);
     if (nok)
     {
       break;
@@ -1373,6 +1721,10 @@ static BOOLEAN jiAssign_list(leftv l, leftv r)
   }
   if (i>li->nr)
   {
+    if (TEST_V_ALLWARN)
+    {
+      Warn("increase list %d -> %d in %s",li->nr,i,my_yylinebuf);
+    }
     li->m=(leftv)omreallocSize(li->m,(li->nr+1)*sizeof(sleftv),(i+1)*sizeof(sleftv));
     memset(&(li->m[li->nr+1]),0,(i-li->nr)*sizeof(sleftv));
     int j=li->nr+1;
@@ -1390,7 +1742,7 @@ static BOOLEAN jiAssign_list(leftv l, leftv r)
     sleftv tmp;
     memset(&tmp,0,sizeof(sleftv));
     tmp.rtyp=DEF_CMD;
-    b=iiAssign(&tmp,r);
+    b=iiAssign(&tmp,r,FALSE);
     ld->CleanUp();
     memcpy(ld,&tmp,sizeof(sleftv));
   }
@@ -1402,13 +1754,13 @@ static BOOLEAN jiAssign_list(leftv l, leftv r)
     memset(&tmp,0,sizeof(sleftv));
     tmp.rtyp=r->Typ();
     tmp.data=(char*)idrecDataInit(r->Typ());
-    b=iiAssign(&tmp,r);
+    b=iiAssign(&tmp,r,FALSE);
     ld->CleanUp();
     memcpy(ld,&tmp,sizeof(sleftv));
   }
   else
   {
-    b=iiAssign(ld,r);
+    b=iiAssign(ld,r,FALSE);
     if (l->e!=NULL) l->e->next=ld->e;
     ld->e=NULL;
   }
@@ -1437,7 +1789,7 @@ static BOOLEAN jiAssign_rec(leftv l, leftv r)
   r1->CleanUp();
   return b;
 }
-BOOLEAN iiAssign(leftv l, leftv r)
+BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
 {
   if (errorreported) return TRUE;
   int ll=l->listLength();
@@ -1455,6 +1807,7 @@ BOOLEAN iiAssign(leftv l, leftv r)
     atKillAll((idhdl)l->data);
     IDFLAG((idhdl)l->data)=0;
     l->attribute=NULL;
+    toplevel=FALSE;
   }
   else if (l->attribute!=NULL)
     atKillAll((idhdl)l);
@@ -1480,7 +1833,7 @@ BOOLEAN iiAssign(leftv l, leftv r)
       }
       if(like_lists)
       {
-        if (traceit&TRACE_ASSIGN) PrintS("assign list[..]=...or similiar\n");
+        if (traceit&TRACE_ASSIGN) PrintS("assign list[..]=...or similar\n");
         if (like_lists==1)
         {
           // check blackbox/newtype type:
@@ -1508,7 +1861,6 @@ BOOLEAN iiAssign(leftv l, leftv r)
         return b;
       }
     }
-    // end of handling elems of list and similar
     if (lt>MAX_TOK)
     {
       blackbox *bb=getBlackboxStuff(lt);
@@ -1517,6 +1869,7 @@ BOOLEAN iiAssign(leftv l, leftv r)
 #endif
       return (bb==NULL) || bb->blackbox_Assign(l,r);
     }
+    // end of handling elems of list and similar
     rl=r->listLength();
     if (rl==1)
     {
@@ -1532,11 +1885,12 @@ BOOLEAN iiAssign(leftv l, leftv r)
       rt=r->Typ();
       /* a = ... */
       if ((lt!=MATRIX_CMD)
-      &&(lt!=INTMAT_CMD)
       &&(lt!=BIGINTMAT_CMD)
+      &&(lt!=CMATRIX_CMD)
+      &&(lt!=INTMAT_CMD)
       &&((lt==rt)||(lt!=LIST_CMD)))
       {
-        b=jiAssign_1(l,r);
+        b=jiAssign_1(l,r,toplevel);
         if (l->rtyp==IDHDL)
         {
           if ((lt==DEF_CMD)||(lt==LIST_CMD))
@@ -1552,15 +1906,16 @@ BOOLEAN iiAssign(leftv l, leftv r)
       }
       if (((lt!=LIST_CMD)
         &&((rt==MATRIX_CMD)
-          ||(rt==INTMAT_CMD)
           ||(rt==BIGINTMAT_CMD)
+          ||(rt==CMATRIX_CMD)
+          ||(rt==INTMAT_CMD)
           ||(rt==INTVEC_CMD)
           ||(rt==MODUL_CMD)))
       ||((lt==LIST_CMD)
         &&(rt==RESOLUTION_CMD))
       )
       {
-        b=jiAssign_1(l,r);
+        b=jiAssign_1(l,r,toplevel);
         if((l->rtyp==IDHDL)&&(l->data!=NULL))
         {
           if ((lt==DEF_CMD) || (lt==LIST_CMD))
@@ -1626,7 +1981,7 @@ BOOLEAN iiAssign(leftv l, leftv r)
     case MAP_CMD:
     {
       // first element in the list sl (r) must be a ring
-      if (((rt == RING_CMD)||(rt == QRING_CMD))&&(r->e==NULL))
+      if ((rt == RING_CMD)&&(r->e==NULL))
       {
         omFree((ADDRESS)IDMAP((idhdl)l->data)->preimage);
         IDMAP((idhdl)l->data)->preimage = omStrDup (r->Fullname());
@@ -1647,7 +2002,7 @@ BOOLEAN iiAssign(leftv l, leftv r)
         break;
       }
       if ((hh->next==NULL)&&(hh->Typ()==IDEAL_CMD))
-        return jiAssign_1(l,hh); /* map-assign: map f=r,i; */
+        return jiAssign_1(l,hh,toplevel); /* map-assign: map f=r,i; */
       //no break, handle the rest like an ideal:
       map_assign=TRUE;
     }
@@ -1657,7 +2012,7 @@ BOOLEAN iiAssign(leftv l, leftv r)
     {
       sleftv t;
       matrix olm = (matrix)l->Data();
-      int rk=olm->rank;
+      int rk;
       char *pr=((map)olm)->preimage;
       BOOLEAN module_assign=(/*l->Typ()*/ lt==MODUL_CMD);
       matrix lm ;
@@ -1669,7 +2024,8 @@ BOOLEAN iiAssign(leftv l, leftv r)
 
       if (lt /*l->Typ()*/==MATRIX_CMD)
       {
-        num=olm->cols()*olm->rows();
+        rk=olm->rows();
+        num=olm->cols()*rk /*olm->rows()*/;
         lm=mpNew(olm->rows(),olm->cols());
         int el;
         if ((traceit&TRACE_ASSIGN) && (num!=(el=exprlist_length(hh))))
@@ -1681,12 +2037,14 @@ BOOLEAN iiAssign(leftv l, leftv r)
       {
         num=exprlist_length(hh);
         lm=(matrix)idInit(num,1);
-        rk=1;
         if (module_assign)
         {
+          rk=0;
           mtyp=MODUL_CMD;
           etyp=VECTOR_CMD;
         }
+        else
+          rk=1;
       }
 
       int ht;
@@ -1778,7 +2136,7 @@ BOOLEAN iiAssign(leftv l, leftv r)
 }
 void jjNormalizeQRingId(leftv I)
 {
-  if ((currQuotient!=NULL) && (!hasFlag(I,FLAG_QRING)))
+  if ((currRing->qideal!=NULL) && (!hasFlag(I,FLAG_QRING)))
   {
     if (I->e==NULL)
     {
@@ -1789,7 +2147,7 @@ void jjNormalizeQRingId(leftv I)
         case MODUL_CMD:
         {
           ideal F=idInit(1,1);
-          ideal II=kNF(F,currQuotient,I0);
+          ideal II=kNF(F,currRing->qideal,I0);
           idDelete(&F);
           if (I->rtyp!=IDHDL)
           {
@@ -1811,38 +2169,19 @@ void jjNormalizeQRingId(leftv I)
     }
   }
 }
-void jjNormalizeQRingP(leftv I)
+void jjNormalizeQRingP(poly &p)
 {
-  if ((currQuotient!=NULL) && (!hasFlag(I,FLAG_QRING)))
+  if((p!=NULL) && (currRing->qideal!=NULL))
   {
-    poly p=(poly)I->Data();
-    if ((I->e==NULL) && (p!=NULL))
-    {
-      ideal F=idInit(1,1);
-      poly II=kNF(F,currQuotient,p);
-      idDelete(&F);
-      if ((I->rtyp==POLY_CMD)
-      || (I->rtyp==VECTOR_CMD))
-      {
-        pDelete(&p);
-        I->data=II;
-      }
-      else if (I->rtyp==IDHDL)
-      {
-        pDelete(&p);
-        idhdl h=(idhdl)I->data;
-        IDPOLY(h)=II;
-        setFlag(h,FLAG_QRING);
-      }
-      else
-      {
-        pDelete(&II);
-      }
-    }
-    setFlag(I,FLAG_QRING);
+    ideal F=idInit(1,1);
+    poly p2=kNF(F,currRing->qideal,p);
+    pNormalize(p2);
+    idDelete(&F);
+    pDelete(&p);
+    p=p2;
   }
 }
-BOOLEAN jjIMPORTFROM(leftv res, leftv u, leftv v)
+BOOLEAN jjIMPORTFROM(leftv, leftv u, leftv v)
 {
   //Print("importfrom %s::%s ->.\n",v->Name(),u->Name() );
   assume(u->Typ()==PACKAGE_CMD);
@@ -1859,7 +2198,7 @@ BOOLEAN jjIMPORTFROM(leftv res, leftv u, leftv v)
     idhdl t=basePack->idroot->get(vn /*v->Name()*/, myynest);
     if (t!=NULL)
     {
-      Warn("redefining `%s`",vn);
+      if (BVERBOSE(V_REDEFINE)) Warn("redefining %s (%s)",vn,my_yylinebuf);
       killhdl(t);
     }
     sleftv tmp_expr;
