@@ -5,32 +5,29 @@
 * ABSTRACT: automatic type conversions
 */
 
+
+
+
 #include <kernel/mod2.h>
 #include <Singular/tok.h>
 #include <Singular/ipid.h>
-#include <kernel/bigintmat.h>
-#include <kernel/intvec.h>
-#include <kernel/options.h>
+#include <misc/intvec.h>
+#include <misc/options.h>
 #include <omalloc/omalloc.h>
-#include <kernel/febase.h>
 #include <kernel/polys.h>
 #include <kernel/ideals.h>
 #include <Singular/subexpr.h>
-#include <kernel/numbers.h>
-#include <kernel/modulop.h>
-#include <kernel/longrat.h>
-#include <kernel/longalg.h>
-#ifdef HAVE_RINGS
-#include <kernel/rmodulon.h>
-#include <kernel/rmodulo2m.h>
-#include <kernel/rintegers.h>
-#endif
-#include <kernel/matpol.h>
-#include <Singular/silink.h>
-#include <kernel/syz.h>
+#include <coeffs/numbers.h>
+#include <coeffs/coeffs.h>
+#include <coeffs/bigintmat.h>
+//#include <polys/ext_fields/longalg.h>
+#include <polys/matpol.h>
+#include <Singular/links/silink.h>
+#include <kernel/GBEngine/syz.h>
 #include <Singular/attrib.h>
-#include <kernel/ring.h>
+#include <polys/monomials/ring.h>
 #include <Singular/ipshell.h>
+#include <Singular/number2.h>
 #include <Singular/ipconv.h>
 
 typedef void *   (*iiConvertProc)(void * data);
@@ -53,9 +50,15 @@ static void * iiI2P(void *data)
 
 static void * iiBI2P(void *data)
 {
-  number n=nInit_bigint((number)data);
-  nlDelete((number *)&data,NULL);
-  poly p=pNSet(n);
+  nMapFunc nMap=n_SetMap(coeffs_BIGINT,currRing->cf);
+  if (nMap==NULL)
+  {
+    Werror("no conversion from bigint to %s", nCoeffString(currRing->cf));
+    return NULL;
+  }
+  number n=nMap((number)data,coeffs_BIGINT,currRing->cf);
+  n_Delete((number *)&data, coeffs_BIGINT);
+  poly p=p_NSet(n, currRing);
   return (void *)p;
 }
 
@@ -68,9 +71,15 @@ static void * iiI2V(void *data)
 
 static void * iiBI2V(void *data)
 {
-  number n=nInit_bigint((number)data);
-  nlDelete((number *)&data,NULL);
-  poly p=pNSet(n);
+  nMapFunc nMap=n_SetMap(coeffs_BIGINT,currRing->cf);
+  if (nMap==NULL)
+  {
+    Werror("no conversion from bigint to %s", nCoeffString(currRing->cf));
+    return NULL;
+  }
+  number n=nMap((number)data,coeffs_BIGINT,currRing->cf);
+  n_Delete((number *)&data, coeffs_BIGINT);
+  poly p=p_NSet(n, currRing);
   if (p!=NULL) pSetComp(p,1);
   return (void *)p;
 }
@@ -85,8 +94,14 @@ static void * iiI2Id(void *data)
 static void * iiBI2Id(void *data)
 {
   ideal I=idInit(1,1);
-  number n=nInit_bigint((number)data);
-  nlDelete((number *)&data,NULL);
+  nMapFunc nMap=n_SetMap(coeffs_BIGINT,currRing->cf);
+  if (nMap==NULL)
+  {
+    Werror("no conversion from bigint to %s", nCoeffString(currRing->cf));
+    return NULL;
+  }
+  number n=nMap((number)data,coeffs_BIGINT,currRing->cf);
+  n_Delete((number *)&data,coeffs_BIGINT);
   poly p=pNSet(n);
   I->m[0]=p;
   return (void *)I;
@@ -101,6 +116,7 @@ static void * iiP2V(void *data)
 static void * iiP2Id(void *data)
 {
   ideal I=idInit(1,1);
+
   if (data!=NULL)
   {
     poly p=(poly)data;
@@ -130,13 +146,13 @@ static void * iiDummy(void *data)
 
 static void * iiMo2Ma(void *data)
 {
-  void *res=idModule2Matrix((ideal)data);
+  void *res=id_Module2Matrix((ideal)data,currRing);
   return res;
 }
 
 static void * iiMa2Mo(void *data)
 {
-  void *res=idMatrix2Module((matrix)data);
+  void *res=id_Matrix2Module((matrix)data,currRing);
   return res;
 }
 
@@ -155,20 +171,135 @@ static void * iiI2N(void *data)
 
 static void * iiI2BI(void *data)
 {
-  number n=nlInit((int)(long)data, NULL /*dummy for nlInit*/);
+  number n=n_Init((int)(long)data, coeffs_BIGINT);
   return (void *)n;
 }
 
+#ifdef SINGULAR_4_2
+static void * iiI2NN(void *data)
+{
+  if (currRing==NULL)
+  {
+    WerrorS("missing basering while converting int to Number");
+    return NULL;
+  }
+  number n=nInit((int)(long)data);
+  number2 nn=(number2)omAlloc(sizeof(*nn));
+  nn->cf=currRing->cf; nn->cf->ref++;
+  nn->n=n;
+  return (void *)nn;
+}
+static void * iiI2CP(void *data)
+{
+  if (currRing==NULL)
+  {
+    WerrorS("missing basering while converting int to Poly");
+    return NULL;
+  }
+  poly n=pISet((int)(long)data);
+  poly2 nn=(poly2)omAlloc(sizeof(*nn));
+  nn->cf=currRing; nn->cf->ref++;
+  nn->n=n;
+  return (void *)nn;
+}
+#endif
+
 static void * iiBI2N(void *data)
 {
   if (currRing==NULL) return NULL;
-  // a bigint is really a number from char 0, with diffrent operations...
-  number n=(number)data;
-  void *r= (void*)nInit_bigint(n);
-  nlDelete(&n,NULL);
-  return r;
+  nMapFunc nMap=n_SetMap(coeffs_BIGINT,currRing->cf);
+  if (nMap==NULL)
+  {
+    Werror("no conversion from bigint to %s", nCoeffString(currRing->cf));
+    return NULL;
+  }
+  number n=nMap((number)data,coeffs_BIGINT,currRing->cf);
+  n_Delete((number *)&data, coeffs_BIGINT);
+  return (void*)n;
 }
 
+#ifdef SINGULAR_4_2
+static void * iiBI2NN(void *data)
+{
+  if (currRing==NULL)
+  {
+    WerrorS("missing basering while converting bigint to Number");
+    return NULL;
+  }
+  nMapFunc nMap=n_SetMap(coeffs_BIGINT,currRing->cf);
+  if (nMap==NULL)
+  {
+    Werror("no conversion from bigint to %s",currRing->cf->cfCoeffString(currRing->cf));
+    return NULL;
+  }
+  number n=nMap((number)data,coeffs_BIGINT,currRing->cf);
+  n_Delete((number *)&data, coeffs_BIGINT);
+  number2 nn=(number2)omAlloc(sizeof(*nn));
+  nn->cf=currRing->cf; nn->cf->ref++;
+  nn->n=n;
+  return (void*)nn;
+}
+static void * iiBI2CP(void *data)
+{
+  if (currRing==NULL)
+  {
+    WerrorS("missing basering while converting bigint to Poly");
+    return NULL;
+  }
+  nMapFunc nMap=n_SetMap(coeffs_BIGINT,currRing->cf);
+  if (nMap==NULL)
+  {
+    Werror("no conversion from bigint to %s",currRing->cf->cfCoeffString(currRing->cf));
+    return NULL;
+  }
+  number n=nMap((number)data,coeffs_BIGINT,currRing->cf);
+  n_Delete((number *)&data, coeffs_BIGINT);
+  poly2 nn=(poly2)omAlloc(sizeof(*nn));
+  nn->cf=currRing; nn->cf->ref++;
+  nn->n=pNSet(n);
+  return (void*)nn;
+}
+static void * iiP2CP(void *data)
+{
+  poly2 nn=(poly2)omAlloc(sizeof(*nn));
+  nn->cf=currRing; nn->cf->ref++;
+  nn->n=(poly)data;
+  return (void*)nn;
+}
+#endif
+
+#ifdef SINGULAR_4_2
+static void * iiNN2N(void *data)
+{
+  number2 d=(number2)data;
+  if ((currRing==NULL)
+  || (currRing->cf!=d->cf))
+  {
+    WerrorS("cannot convert: incompatible");
+    return NULL;
+  }
+  number n = n_Copy(d->n, d->cf);
+  n2Delete(d);
+  return (void*)n;
+}
+#endif
+
+#ifdef SINGULAR_4_2
+static void * iiNN2P(void *data)
+{
+  number2 d=(number2)data;
+  if ((currRing==NULL)
+  || (currRing->cf!=d->cf))
+  {
+    WerrorS("cannot convert: incompatible");
+    return NULL;
+  }
+  number n = n_Copy(d->n, d->cf);
+  n2Delete(d);
+  return (void*)p_NSet(n,currRing);
+}
+#endif
+
 static void * iiIm2Ma(void *data)
 {
   int i, j;
@@ -189,19 +320,11 @@ static void * iiIm2Ma(void *data)
 static void * iiIm2Bim(void *data)
 {
   intvec *iv=(intvec*)data;
-  void *r=(void *)iv2bim((intvec*)data);
+  void *r=(void *)iv2bim(iv,coeffs_BIGINT);
   delete iv;
   return r;
 }
 
-static void * iiBim2Im(void *data)
-{
-  bigintmat *b=(bigintmat*)data;
-  void *r=(void *)bim2iv((bigintmat*)data);
-  delete b;
-  return r;
-}
-
 static void * iiN2P(void *data)
 {
   poly p=NULL;
@@ -239,13 +362,6 @@ static void * iiS2Link(void *data)
   return (void *)l;
 }
 
-/*
-static void * iiR2L(void * data)
-{
-  syStrategy tmp=(syStrategy)data;
-  return  (void *)syConvRes(tmp,TRUE);
-}
-*/
 static void iiR2L_l(leftv out, leftv in)
 {
   int add_row_shift = 0;
@@ -257,9 +373,18 @@ static void iiR2L_l(leftv out, leftv in)
   out->data=(void *)syConvRes(tmp,TRUE,add_row_shift);
 }
 
-static void * iiL2R(void * data)
+static void iiL2R(leftv out, leftv in)
 {
-  return (void *)syConvList((lists)data,TRUE);
+  int add_row_shift = 0;
+  lists l=(lists)in->Data();
+  intvec *ww=NULL;
+  if (l->nr>=0) ww=(intvec *)atGet(&(l->m[0]),"isHomog",INTVEC_CMD);
+  out->data=(void *)syConvList(l);
+  if (ww!=NULL)
+  {
+    intvec *weights=ivCopy(ww);
+    atSet(out,"isHomog",weights,INTVEC_CMD);
+  }
 }
 
 //
@@ -273,7 +398,7 @@ static void * iiL2R(void * data)
 * try to convert 'input' of type 'inputType' to 'output' of type 'outputType'
 * return FALSE on success
 */
-BOOLEAN iiConvert (int inputType, int outputType, int index, leftv input, leftv output)
+BOOLEAN iiConvert (int inputType, int outputType, int index, leftv input, leftv output,const struct sConvertTypes *dConvertTypes)
 {
   memset(output,0,sizeof(sleftv));
   if ((inputType==outputType)
@@ -297,7 +422,7 @@ BOOLEAN iiConvert (int inputType, int outputType, int index, leftv input, leftv
       else if (input->name!=NULL)
       {
         if (input->rtyp==ALIAS_CMD)
-          output->name=omStrDup(input->name);
+        output->name=omStrDup(input->name);
         else
         {
           output->name=input->name;
@@ -325,13 +450,21 @@ BOOLEAN iiConvert (int inputType, int outputType, int index, leftv input, leftv
           }
           else if(pIsConstant((poly)input->data))
           {
-            output->name=nName(pGetCoeff((poly)input->data));
+            StringSetS("");
+            number n=(pGetCoeff((poly)input->data));
+            n_Write(n, currRing->cf);
+            (pGetCoeff((poly)input->data))=n;
+            output->name=StringEndS();
           }
         }
       }
       else if ((input->rtyp==NUMBER_CMD) && (input->name==NULL))
       {
-        output->name=nName((number)input->data);
+        StringSetS("");
+        number n=(number)input->data;
+        n_Write(n, currRing->cf);
+        input->data=(void*)n;
+        output->name=StringEndS();
       }
       else
       {
@@ -342,7 +475,7 @@ BOOLEAN iiConvert (int inputType, int outputType, int index, leftv input, leftv
     }
     output->next=input->next;
     input->next=NULL;
-    return FALSE;
+    return errorreported;
   }
   if (index!=0) /* iiTestConvert does not returned 'failure' */
   {
@@ -375,6 +508,7 @@ BOOLEAN iiConvert (int inputType, int outputType, int index, leftv input, leftv
       {
         return TRUE;
       }
+      if (errorreported) return TRUE;
       output->next=input->next;
       input->next=NULL;
   //if (outputType==MATRIX_CMD) Print("convert %d -> matrix\n",inputType);
@@ -388,7 +522,7 @@ BOOLEAN iiConvert (int inputType, int outputType, int index, leftv input, leftv
 * try to convert 'inputType' in 'outputType'
 * return 0 on failure, an index (<>0) on success
 */
-int iiTestConvert (int inputType, int outputType)
+int iiTestConvert (int inputType, int outputType,const struct sConvertTypes *dConvertTypes)
 {
   if ((inputType==outputType)
   || (outputType==DEF_CMD)
@@ -397,9 +531,12 @@ int iiTestConvert (int inputType, int outputType)
   {
     return -1;
   }
+  if (inputType==UNKNOWN) return 0;
 
   if ((currRing==NULL) && (outputType>BEGIN_RING) && (outputType<END_RING))
     return 0;
+  //if ((currRing==NULL) && (outputType==CNUMBER_CMD))
+  //  return 0;
 
   // search the list
   int i=0;
@@ -414,7 +551,7 @@ int iiTestConvert (int inputType, int outputType)
     }
     i++;
   }
-  //Print("test convert %d to %d (%s -> %s):0\n",inputType,outputType,
-  // Tok2Cmdname(inputType), Tok2Cmdname(outputType));
+  //Print("test convert %d to %d (%s -> %s):0, tested:%d\n",inputType,outputType,
+  // Tok2Cmdname(inputType), Tok2Cmdname(outputType),i);
   return 0;
 }
