@@ -1,45 +1,61 @@
 /****************************************
 *  Computer Algebra System SINGULAR     *
 ****************************************/
+
 /*
 * ABSTRACT: identfier handling
 */
 
-#include <string.h>
+
+
+
 
 #include <kernel/mod2.h>
-#include <Singular/static.h>
+
 #include <omalloc/omalloc.h>
-#include <Singular/tok.h>
-#include <kernel/options.h>
-#include <Singular/ipshell.h>
-#include <kernel/intvec.h>
-#include <kernel/bigintmat.h>
-#include <kernel/febase.h>
-#include <kernel/numbers.h>
-#include <kernel/longrat.h>
+
+#include <misc/options.h>
+#include <misc/intvec.h>
+
+#include <coeffs/numbers.h>
+#include <coeffs/bigintmat.h>
+
+#include <polys/matpol.h>
+#include <polys/monomials/ring.h>
+
 #include <kernel/polys.h>
-#include <kernel/ring.h>
 #include <kernel/ideals.h>
-#include <kernel/matpol.h>
+#include <kernel/GBEngine/syz.h>
+
+#include <Singular/tok.h>
+#include <Singular/ipshell.h>
+#include <Singular/fevoices.h>
 #include <Singular/lists.h>
 #include <Singular/attrib.h>
-#include <Singular/silink.h>
-#include <kernel/syz.h>
+#include <Singular/links/silink.h>
 #include <Singular/ipid.h>
 #include <Singular/blackbox.h>
+#include <Singular/number2.h>
 
+#ifdef SINGULAR_4_1
+#include <Singular/number2.h>
+#endif
 #ifdef HAVE_DYNAMIC_LOADING
-#include <kernel/mod_raw.h>
+#include <polys/mod_raw.h>
 #endif /* HAVE_DYNAMIC_LOADING */
 
+#include <string.h>
+
 omBin sip_command_bin = omGetSpecBin(sizeof(sip_command));
 omBin sip_package_bin = omGetSpecBin(sizeof(sip_package));
 //omBin ip_package_bin = omGetSpecBin(sizeof(ip_package));
 omBin idrec_bin = omGetSpecBin(sizeof(idrec));
 
+coeffs coeffs_BIGINT;
+
+FILE   *feFilePending; /*temp. storage for grammar.y */
+
 proclevel *procstack=NULL;
-#define TEST
 //idhdl idroot = NULL;
 
 idhdl currPackHdl = NULL;
@@ -47,8 +63,6 @@ idhdl basePackHdl = NULL;
 package currPack =NULL;
 package basePack =NULL;
 idhdl currRingHdl = NULL;
-ring  currRing = NULL;
-ideal currQuotient = NULL;
 const char* iiNoName="_";
 
 void paCleanUp(package pack);
@@ -58,17 +72,16 @@ void paCleanUp(package pack);
 int iiS2I(const char *s)
 {
   int i;
-  int c;
   i=s[0];
-  if ((c=s[1])!='\0')
+  if (s[1]!='\0')
   {
-    i=(i<<8)+c;
-    if ((c=s[2])!='\0')
+    i=(i<<8)+s[1];
+    if (s[2]!='\0')
     {
-      i=(i<<8)+c;
-      if ((c=s[3])!='\0')
+      i=(i<<8)+s[2];
+      if (s[3]!='\0')
       {
-        i=(i<<8)+c;
+        i=(i<<8)+s[3];
       }
     }
   }
@@ -121,15 +134,22 @@ void *idrecDataInit(int t)
   switch (t)
   {
     //the type with init routines:
+#ifdef SINGULAR_4_2
+    case CNUMBER_CMD:
+      return (void*)n2Init(0,NULL);
+    case CPOLY_CMD:
+      return (void*)p2Init(0,NULL);
+    case CMATRIX_CMD:
+#endif
+    case BIGINTMAT_CMD:
+      return (void *)new bigintmat();
     case INTVEC_CMD:
     case INTMAT_CMD:
       return (void *)new intvec();
-    case BIGINTMAT_CMD:
-      return (void *)new bigintmat();
     case NUMBER_CMD:
       return (void *) nInit(0);
     case BIGINT_CMD:
-      return (void *) nlInit(0, NULL /* dummy for nlInit*/);
+      return (void *) n_Init(0, coeffs_BIGINT);
     case IDEAL_CMD:
     case MODUL_CMD:
     case MATRIX_CMD:
@@ -152,14 +172,27 @@ void *idrecDataInit(int t)
     case LINK_CMD:
       return (void*) omAlloc0Bin(sip_link_bin);
     case RING_CMD:
-      return (void*) omAlloc0Bin(sip_sring_bin);
+      return NULL;
     case PACKAGE_CMD:
-      return (void*) omAlloc0Bin(sip_package_bin);
+    {
+      package pa=(package)omAlloc0Bin(sip_package_bin);
+      pa->language=LANG_NONE;
+      pa->loaded = FALSE;
+      return (void*)pa;
+    }
     case PROC_CMD:
-      return (void *) omAlloc0Bin(procinfo_bin);
+    {
+      procinfov pi=(procinfov)omAlloc0Bin(procinfo_bin);
+      pi->ref=1;
+      pi->language=LANG_NONE;
+      return (void*)pi;
+    }
     case RESOLUTION_CMD:
       return  (void *)omAlloc0(sizeof(ssyStrategy));
     //other types: without init (int,script,poly,def,package)
+    #ifdef SINGULAR_4_1
+    case CRING_CMD:
+    #endif
     case INT_CMD:
     case DEF_CMD:
     case POLY_CMD:
@@ -178,7 +211,7 @@ void *idrecDataInit(int t)
              return (void *)bb->blackbox_Init(bb);
         }
         else
-          Werror("unknown type %d",t);
+          Werror("unknown type in idrecDataInit:%d",t);
         break;
       }
   }
@@ -191,6 +224,7 @@ idhdl idrec::set(const char * s, int level, int t, BOOLEAN init)
   IDID(h)   = s;
   IDTYP(h)  = t;
   IDLEV(h)  = level;
+  IDNEXT(h) = this;
   BOOLEAN at_start=(this==IDROOT);
   h->id_i=iiS2I(s);
   if (init)
@@ -206,32 +240,22 @@ idhdl idrec::set(const char * s, int level, int t, BOOLEAN init)
       // IDRING(h)=rCopy(currRing);
       /* QRING_CMD is ring dep => currRing !=NULL */
     }
-    else
 #endif
-    if (t == PROC_CMD)
-    {
-      IDPROC(h)->language=LANG_NONE;
-    }
-    else if (t == PACKAGE_CMD)
-    {
-      IDPACKAGE(h)->language=LANG_NONE;
-      IDPACKAGE(h)->loaded = FALSE;
-    }
   }
-  if (at_start) IDNEXT(h) = IDROOT;
-  else          IDNEXT(h) = this;
   // --------------------------------------------------------
+  if (at_start)
+    IDNEXT(h) = IDROOT;
   return  h;
 }
 
-char * idrec::String()
+char * idrec::String(BOOLEAN typed)
 {
   sleftv tmp;
   memset(&tmp,0,sizeof(sleftv));
   tmp.rtyp=IDTYP(this);
   tmp.data=IDDATA(this);
   tmp.name=IDID(this);
-  return tmp.String();
+  return tmp.String(NULL, typed);
 }
 
 idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN search)
@@ -240,6 +264,7 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
   if (root==NULL) return NULL;
   idhdl h;
   s=omStrDup(s);
+  // idhdl *save_root=root;
   if (t==PACKAGE_CMD)
   {
     if (root!=&(basePack->idroot))
@@ -254,15 +279,21 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
     {
       if ((IDTYP(h) == t)||(t==DEF_CMD))
       {
-        if ((IDTYP(h)==PACKAGE_CMD)
-        && (strcmp(s,"Top")==0))
+        if (IDTYP(h)==PACKAGE_CMD)
         {
-          goto errlabel;
+          if (strcmp(s,"Top")==0)
+          {
+            goto errlabel;
+          }
+          else return *root;
+        }
+        else
+        {
+          if (BVERBOSE(V_REDEFINE))
+            Warn("redefining %s (%s)",s,my_yylinebuf);
+          if (s==IDID(h)) IDID(h)=NULL;
+          killhdl2(h,root,currRing);
         }
-        if (BVERBOSE(V_REDEFINE))
-          Warn("redefining %s **",s);
-        if (s==IDID(h)) IDID(h)=NULL;
-        killhdl2(h,root,currRing);
       }
       else
         goto errlabel;
@@ -278,8 +309,8 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
         if ((IDTYP(h) == t)||(t==DEF_CMD))
         {
           if (BVERBOSE(V_REDEFINE))
-            Warn("redefining %s **",s);
-          IDID(h)=NULL;
+            Warn("redefining %s (%s)",s,my_yylinebuf);
+          if (s==IDID(h)) IDID(h)=NULL;
           killhdl2(h,&currRing->idroot,currRing);
         }
         else
@@ -297,7 +328,7 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
         if ((IDTYP(h) == t)||(t==DEF_CMD))
         {
           if (BVERBOSE(V_REDEFINE))
-            Warn("redefining `%s` **",s);
+            Warn("redefining %s (%s)",s,my_yylinebuf);
           if (s==IDID(h)) IDID(h)=NULL;
           killhdl2(h,&IDROOT,NULL);
         }
@@ -307,7 +338,7 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
     }
   }
   *root = (*root)->set(s, lev, t, init);
-#ifndef NDEBUG
+#ifndef SING_NDEBUG
   checkall();
 #endif
   return *root;
@@ -343,13 +374,13 @@ void killid(const char * id, idhdl * ih)
     killhdl2(h,ih,currRing);
   }
   else
-    Werror("kill what ?");
+    WerrorS("kill what ?");
 }
 
 void killhdl(idhdl h, package proot)
 {
   int t=IDTYP(h);
-  if (((BEGIN_RING<t) && (t<END_RING) && (t!=QRING_CMD))
+  if (((BEGIN_RING<t) && (t<END_RING))
   || ((t==LIST_CMD) && (lRingDependend((lists)IDDATA(h)))))
     killhdl2(h,&currRing->idroot,currRing);
   else
@@ -382,6 +413,14 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
   //printf("kill %s, id %x, typ %d lev: %d\n",IDID(h),(int)IDID(h),IDTYP(h),IDLEV(h));
   idhdl hh;
 
+  if (TEST_V_ALLWARN
+  && (IDLEV(h)!=myynest)
+  &&(IDLEV(h)==0))
+  {
+    if (((*ih)==basePack->idroot)
+    || ((currRing!=NULL)&&((*ih)==currRing->idroot)))
+      Warn("kill global `%s` at line >>%s<<\n",IDID(h),my_yylinebuf);
+  }
   if (h->attribute!=NULL)
   {
     //h->attribute->killAll(r); MEMORY LEAK!
@@ -418,7 +457,7 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
     if (currPackHdl==h) currPackHdl=packFindHdl(currPack);
     iiCheckPack(currPack);
   }
-  else if ((IDTYP(h)==RING_CMD)||(IDTYP(h)==QRING_CMD))
+  else if (IDTYP(h)==RING_CMD)
     rKill(h);
   else if (IDDATA(h)!=NULL)
     s_internalDelete(IDTYP(h),IDDATA(h),r);
@@ -456,7 +495,7 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
   omFreeBin((ADDRESS)h, idrec_bin);
 }
 
-idhdl ggetid(const char *n, BOOLEAN local, idhdl *packhdl)
+idhdl ggetid(const char *n, BOOLEAN /*local*/, idhdl *packhdl)
 {
   idhdl h = IDROOT->get(n,myynest);
   idhdl h2=NULL;
@@ -574,7 +613,7 @@ void  ipMoveId(idhdl tomove)
 {
   if ((currRing!=NULL)&&(tomove!=NULL))
   {
-    if (((QRING_CMD!=IDTYP(tomove)) && RingDependend(IDTYP(tomove)))
+    if (RingDependend(IDTYP(tomove))
     || ((IDTYP(tomove)==LIST_CMD) && (lRingDependend(IDLIST(tomove)))))
     {
       /*move 'tomove' to ring id's*/
@@ -601,7 +640,7 @@ const char * piProcinfo(procinfov pi, const char *request)
       case LANG_SINGULAR: return "singular"; break;
       case LANG_C:        return "object";   break;
       case LANG_NONE:     return "none";     break;
-      default:            return "unknow language";
+      default:            return "unknown language";
     }
   }
   else if (strcmp(request, "ref")      == 0)
@@ -613,11 +652,21 @@ const char * piProcinfo(procinfov pi, const char *request)
   return "??";
 }
 
-void piCleanUp(procinfov pi)
+BOOLEAN piKill(procinfov pi)
 {
   (pi->ref)--;
   if (pi->ref <= 0)
   {
+    Voice *p=currentVoice;
+    while (p!=NULL)
+    {
+      if (p->pi==pi && pi->ref <= 1)
+      {
+        Warn("`%s` in use, can not be killed",pi->procname);
+        return TRUE;
+      }
+      p=p->next;
+    }
     if (pi->libname != NULL) // OB: ????
       omFree((ADDRESS)pi->libname);
     if (pi->procname != NULL) // OB: ????
@@ -633,24 +682,8 @@ void piCleanUp(procinfov pi)
     }
     memset((void *) pi, 0, sizeof(procinfo));
     pi->language=LANG_NONE;
-  }
-}
-
-BOOLEAN piKill(procinfov pi)
-{
-  Voice *p=currentVoice;
-  while (p!=NULL)
-  {
-    if (p->pi==pi && pi->ref <= 1)
-    {
-      Warn("`%s` in use, can not be killed",pi->procname);
-      return TRUE;
-    }
-    p=p->next;
-  }
-  piCleanUp(pi);
-  if (pi->ref <= 0)
     omFreeBin((ADDRESS)pi,  procinfo_bin);
+  }
   return FALSE;
 }
 
@@ -678,8 +711,6 @@ void proclevel::push(char *n)
 {
   //Print("push %s\n",n);
   proclevel *p=(proclevel*)omAlloc0(sizeof(proclevel));
-  p->cRing=currRing;
-  p->cRingHdl=currRingHdl;
   p->name=n;
   p->cPackHdl=currPackHdl;
   p->cPack=currPack;
@@ -694,7 +725,7 @@ void proclevel::pop()
   //if (r==NULL) Print("set ring to NULL at lev %d(%s)\n",myynest,name);
   //::currRingHdl=this->currRingHdl;
   //if((::currRingHdl==NULL)||(IDRING(::currRingHdl)!=(::currRing)))
-  //  ::currRingHdl=rFindHdl(::currRing,NULL);
+  //  ::currRingHdl=rFindHdl(::currRing,NULL,NULL);
   //Print("restore pack=%s,1.obj=%s\n",IDID(currPackHdl),IDID(currPack->idroot));
   currPackHdl=this->cPackHdl;
   currPack=this->cPack;
@@ -716,3 +747,88 @@ idhdl packFindHdl(package r)
   }
   return NULL;
 }
+
+BOOLEAN iiAlias(leftv p)
+{
+  if (iiCurrArgs==NULL)
+  {
+    Werror("not enough arguments for proc %s",VoiceName());
+    p->CleanUp();
+    return TRUE;
+  }
+  leftv h=iiCurrArgs;
+  iiCurrArgs=h->next;
+  h->next=NULL;
+  if (h->rtyp!=IDHDL)
+  {
+    BOOLEAN res=iiAssign(p,h);
+    h->CleanUp();
+    omFreeBin((ADDRESS)h, sleftv_bin);
+    return res;
+  }
+  if ((h->Typ()!=p->Typ()) &&(p->Typ()!=DEF_CMD))
+  {
+    WerrorS("type mismatch");
+    return TRUE;
+  }
+  idhdl pp=(idhdl)p->data;
+  switch(pp->typ)
+  {
+#ifdef SINGULAR_4_1
+      case CRING_CMD:
+        nKillChar((coeffs)pp);
+        break;
+#endif
+      case DEF_CMD:
+      case INT_CMD:
+        break;
+      case INTVEC_CMD:
+      case INTMAT_CMD:
+         delete IDINTVEC(pp);
+         break;
+      case NUMBER_CMD:
+         nDelete(&IDNUMBER(pp));
+         break;
+      case BIGINT_CMD:
+         n_Delete(&IDNUMBER(pp),coeffs_BIGINT);
+         break;
+      case MAP_CMD:
+         {
+           map im = IDMAP(pp);
+           omFree((ADDRESS)im->preimage);
+         }
+         // continue as ideal:
+      case IDEAL_CMD:
+      case MODUL_CMD:
+      case MATRIX_CMD:
+          idDelete(&IDIDEAL(pp));
+         break;
+      case PROC_CMD:
+      case RESOLUTION_CMD:
+      case STRING_CMD:
+         omFree((ADDRESS)IDSTRING(pp));
+         break;
+      case LIST_CMD:
+         IDLIST(pp)->Clean();
+         break;
+      case LINK_CMD:
+         omFreeBin(IDLINK(pp),sip_link_bin);
+         break;
+       // case ring: cannot happen
+       default:
+         Werror("unknown type %d",p->Typ());
+         return TRUE;
+  }
+  pp->typ=ALIAS_CMD;
+  IDDATA(pp)=(char*)h->data;
+  int eff_typ=h->Typ();
+  if ((RingDependend(eff_typ))
+  || ((eff_typ==LIST_CMD) && (lRingDependend((lists)h->Data()))))
+  {
+    ipSwapId(pp,IDROOT,currRing->idroot);
+  }
+  h->CleanUp();
+  omFreeBin((ADDRESS)h, sleftv_bin);
+  return FALSE;
+}
+
