@@ -5,27 +5,33 @@
 * ABSTRACT: interpreter: LIB and help
 */
 
+
+
+
+
+#include <kernel/mod2.h>
+
+#include <Singular/tok.h>
+#include <misc/options.h>
+#include <Singular/ipid.h>
+#include <omalloc/omalloc.h>
+#include <polys/monomials/ring.h>
+#include <Singular/subexpr.h>
+#include <Singular/ipid.h>
+#include <Singular/ipshell.h>
+#include <Singular/fevoices.h>
+#include <Singular/lists.h>
+
 //#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include <sys/stat.h>
 
-#include <kernel/mod2.h>
-#include <Singular/static.h>
-#include <Singular/tok.h>
-#include <kernel/options.h>
-#include <Singular/ipid.h>
-#include <omalloc/omalloc.h>
-#include <kernel/febase.h>
-#include <kernel/ring.h>
-#include <Singular/subexpr.h>
-#include <Singular/ipshell.h>
-#include <Singular/lists.h>
 
 #if SIZEOF_LONG == 8
 #define SI_MAX_NEST 500
-#elif defined(ix86_Win)
+#elif defined(__CYGWIN__)
 #define SI_MAX_NEST 480
 #else
 #define SI_MAX_NEST 1000
@@ -33,7 +39,7 @@
 
 #if defined(ix86Mac_darwin) || defined(x86_64Mac_darwin) || defined(ppcMac_darwin)
 #  define MODULE_SUFFIX bundle
-#elif defined(ix86_Win)
+#elif defined(__CYGWIN__)
 #  define MODULE_SUFFIX dll
 #else
 #  define MODULE_SUFFIX so
@@ -43,7 +49,7 @@
 
 
 #ifdef HAVE_DYNAMIC_LOADING
-BOOLEAN load_modules(char *newlib, char *fullname, BOOLEAN autoexport);
+BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport);
 #endif
 
 #ifdef HAVE_LIBPARSER
@@ -52,12 +58,11 @@ BOOLEAN load_modules(char *newlib, char *fullname, BOOLEAN autoexport);
 procinfo *iiInitSingularProcinfo(procinfov pi, const char *libname,
               const char *procname, int line, long pos, BOOLEAN pstatic=FALSE);
 #endif /* HAVE_LIBPARSER */
-#define NS_LRING (procstack->cRing)
 
 extern int iiArithAddCmd(const char *szName, short nAlias, short nTokval,
                          short nToktype, short nPos);
 
-#include <kernel/mod_raw.h>
+#include <Singular/mod_lib.h>
 
 #ifdef HAVE_LIBPARSER
 void yylprestart (FILE *input_file );
@@ -147,7 +152,17 @@ char * iiProcArgs(char *e,BOOLEAN withParenth)
     args_found=FALSE;
     s=e; // set s to the starting point of the arg
          // and search for the end
-    while(*s==' ') s++; e=s;
+    // skip leading spaces:
+    loop
+    {
+      if ((*s==' ')||(*s=='\t'))
+        s++;
+      else if ((*s=='\n')&&(*(s+1)==' '))
+        s+=2;
+      else // start of new arg or \0 or )
+        break;
+    }
+    e=s;
     while ((*e!=',')
     &&((par!=0) || (*e!=')'))
     &&(*e!='\0'))
@@ -278,7 +293,7 @@ char* iiGetLibProcBuffer(procinfo *pi, int part )
       return NULL; // example part does not exist
     // load example
     fseek(fp, pi->data.s.example_start, SEEK_SET);
-    char *dummy=fgets(buf, sizeof(buf), fp); // skip line with "example"
+    /*char *dummy=*/ (void) fgets(buf, sizeof(buf), fp); // skip line with "example"
     procbuflen = pi->data.s.proc_end - pi->data.s.example_start - strlen(buf);
     //Print("Example=%ld-%ld=%d\n", pi->data.s.proc_end,
     //  pi->data.s.example_start, procbuflen);
@@ -292,39 +307,24 @@ char* iiGetLibProcBuffer(procinfo *pi, int part )
   }
   return NULL;
 }
-#ifndef LIBSINGULAR
-// see below:
-struct soptionStruct
-{
-  const char * name;
-  unsigned   setval;
-  unsigned   resetval;
-};
-extern struct soptionStruct optionStruct[];
-extern struct soptionStruct verboseStruct[];
-#endif
 
 BOOLEAN iiAllStart(procinfov pi, char *p,feBufferTypes t, int l)
 {
+  // see below:
+  BITSET save1=si_opt_1;
+  BITSET save2=si_opt_2;
   newBuffer( omStrDup(p /*pi->data.s.body*/), t /*BT_proc*/,
                pi, l );
-  #ifndef LIBSINGULAR
-  // see below:
-  BITSET save1=(test & ~TEST_RINGDEP_OPTS);
-  BITSET save2=verbose;
-  #endif
   BOOLEAN err=yyparse();
   if (sLastPrinted.rtyp!=0)
   {
     sLastPrinted.CleanUp();
   }
-  #ifndef LIBSINGULAR
   // the access to optionStruct and verboseStruct do not work
   // on x86_64-Linux for pic-code
-  BITSET save11= ( test & ~TEST_RINGDEP_OPTS);
   if ((TEST_V_ALLWARN) &&
   (t==BT_proc) &&
-  ((save1!=save11)||(save2!=verbose)) &&
+  ((save1!=si_opt_1)||(save2!=si_opt_2)) &&
   (pi->libname!=NULL) && (pi->libname[0]!='\0'))
   {
     if ((pi->libname!=NULL) && (pi->libname[0]!='\0'))
@@ -334,12 +334,12 @@ BOOLEAN iiAllStart(procinfov pi, char *p,feBufferTypes t, int l)
     int i;
     for (i=0; optionStruct[i].setval!=0; i++)
     {
-      if ((optionStruct[i].setval & save11)
+      if ((optionStruct[i].setval & si_opt_1)
       && (!(optionStruct[i].setval & save1)))
       {
           Print(" +%s",optionStruct[i].name);
       }
-      if (!(optionStruct[i].setval & save11)
+      if (!(optionStruct[i].setval & si_opt_1)
       && ((optionStruct[i].setval & save1)))
       {
           Print(" -%s",optionStruct[i].name);
@@ -347,21 +347,19 @@ BOOLEAN iiAllStart(procinfov pi, char *p,feBufferTypes t, int l)
     }
     for (i=0; verboseStruct[i].setval!=0; i++)
     {
-      if ((verboseStruct[i].setval & verbose)
+      if ((verboseStruct[i].setval & si_opt_2)
       && (!(verboseStruct[i].setval & save2)))
       {
           Print(" +%s",verboseStruct[i].name);
       }
-      if (!(verboseStruct[i].setval & verbose)
+      if (!(verboseStruct[i].setval & si_opt_2)
       && ((verboseStruct[i].setval & save2)))
       {
           Print(" -%s",verboseStruct[i].name);
       }
     }
     PrintLn();
-  //  PrintS(p);
   }
-  #endif
   return err;
 }
 /*2
@@ -419,10 +417,6 @@ BOOLEAN iiPStart(idhdl pn, sleftv  * v)
   {
     err=iiAllStart(pi,pi->data.s.body,BT_proc,pi->data.s.body_lineno-(v!=NULL));
 
-#ifdef USE_IILOCALRING
-#if 0
-  if(procstack->cRing != iiLocalRing[myynest]) Print("iiMake_proc: 1 ring not saved procs:%x, iiLocal:%x\n",procstack->cRing, iiLocalRing[myynest]);
-#endif
     if (iiLocalRing[myynest-1] != currRing)
     {
       if (iiRETURNEXPR.RingDependend())
@@ -456,48 +450,9 @@ BOOLEAN iiPStart(idhdl pn, sleftv  * v)
       rSetHdl(rFindHdl(currRing,NULL));
       iiLocalRing[myynest-1]=NULL;
     }
-#else /* USE_IILOCALRING */
-    if (procstack->cRing != currRing)
-    {
-      //if (procstack->cRingHdl!=NULL)
-      //Print("procstack:%s,",IDID(procstack->cRingHdl));
-      //if (currRingHdl!=NULL)
-      //Print(" curr:%s\n",IDID(currRingHdl));
-      //Print("pr:%x, curr: %x\n",procstack->cRing,currRing);
-      if (iiRETURNEXPR.RingDependend())
-      {
-        //idhdl hn;
-        char *n;
-        char *o;
-        if (procstack->cRing!=NULL)
-        {
-          //PrintS("reset ring\n");
-          procstack->cRingHdl=rFindHdl(procstack->cRing,NULL);
-          o=IDID(procstack->cRingHdl);
-          currRing=procstack->cRing;
-          currRingHdl=procstack->cRingHdl;
-        }
-        else                            o="none";
-        if (currRing!=NULL)             n=IDID(currRingHdl);
-        else                            n="none";
-        if (currRing==NULL)
-        {
-          Werror("ring change during procedure call: %s -> %s",o,n);
-          iiRETURNEXPR.CleanUp();
-          err=TRUE;
-        }
-      }
-      if (procstack->cRingHdl!=NULL)
-      {
-        rSetHdl(procstack->cRingHdl);
-      }
-      else
-      { currRingHdl=NULL; currRing=NULL; }
-    }
-#endif /* USE_IILOCALRING */
     //Print("kill locals for %s (level %d)\n",IDID(pn),myynest);
     killlocals(myynest);
-#ifndef NDEBUG
+#ifndef SING_NDEBUG
     checkall();
 #endif
     //Print("end kill locals for %s (%d)\n",IDID(pn),myynest);
@@ -512,9 +467,7 @@ BOOLEAN iiPStart(idhdl pn, sleftv  * v)
   return err;
 }
 
-#ifdef USE_IILOCALRING
 ring    *iiLocalRing;
-#endif
 sleftv  iiRETURNEXPR;
 int     iiRETURNEXPR_len=0;
 
@@ -522,7 +475,6 @@ int     iiRETURNEXPR_len=0;
 static void iiShowLevRings()
 {
   int i;
-#ifdef USE_IILOCALRING
   for (i=0;i<=myynest;i++)
   {
     Print("lev %d:",i);
@@ -530,19 +482,6 @@ static void iiShowLevRings()
     else                      Print("%lx",(long)iiLocalRing[i]);
     PrintLn();
   }
-#endif
-#if  0
-  i=myynest;
-  proclevel *p=procstack;
-  while (p!=NULL)
-  {
-    Print("lev %d:",i);
-    if (p->cRingHdl==NULL) PrintS("NULL");
-    else                   Print("%s",IDID(p->cRingHdl));
-    PrintLn();
-    p=p->next;
-  }
-#endif
   if (currRing==NULL) PrintS("curr:NULL\n");
   else                Print ("curr:%lx\n",(long)currRing);
 }
@@ -552,12 +491,10 @@ static void iiCheckNest()
 {
   if (myynest >= iiRETURNEXPR_len-1)
   {
-#ifdef USE_IILOCALRING
     iiLocalRing=(ring *)omreallocSize(iiLocalRing,
                                    iiRETURNEXPR_len*sizeof(ring),
                                    (iiRETURNEXPR_len+16)*sizeof(ring));
     memset(&(iiLocalRing[iiRETURNEXPR_len]),0,16*sizeof(ring));
-#endif
     iiRETURNEXPR_len+=16;
   }
 }
@@ -572,10 +509,9 @@ BOOLEAN iiMake_proc(idhdl pn, package pack, sleftv* sl)
     return TRUE;
   }
   iiCheckNest();
-#ifdef USE_IILOCALRING
   iiLocalRing[myynest]=currRing;
   //Print("currRing(%d):%s(%x) in %s\n",myynest,IDID(currRingHdl),currRing,IDID(pn));
-#endif
+  iiRETURNEXPR.Init();
   procstack->push(pi->procname);
   if ((traceit&TRACE_SHOW_PROC)
   || (pi->trace_flag&TRACE_SHOW_PROC))
@@ -624,7 +560,7 @@ BOOLEAN iiMake_proc(idhdl pn, package pack, sleftv* sl)
     if (traceit&TRACE_SHOW_LINENO) PrintLn();
     Print("leaving %-*.*s %s (level %d)\n",myynest*2,myynest*2," ",IDID(pn),myynest);
   }
-  //char *n="NULL";
+  //const char *n="NULL";
   //if (currRingHdl!=NULL) n=IDID(currRingHdl);
   //Print("currRing(%d):%s(%x) after %s\n",myynest,n,currRing,IDID(pn));
 #ifdef RDEBUG
@@ -659,9 +595,7 @@ BOOLEAN iiEStart(char* example, procinfo *pi)
 
   iiCheckNest();
   procstack->push(example);
-#ifdef USE_IILOCALRING
   iiLocalRing[myynest]=currRing;
-#endif
   if (traceit&TRACE_SHOW_PROC)
   {
     if (traceit&TRACE_SHOW_LINENO) printf("\n");
@@ -679,7 +613,6 @@ BOOLEAN iiEStart(char* example, procinfo *pi)
     if (traceit&TRACE_SHOW_LINENO) printf("\n");
     printf("leaving  -example- (level %d)\n",myynest);
   }
-#ifdef USE_IILOCALRING
   if (iiLocalRing[myynest] != currRing)
   {
     if (iiLocalRing[myynest]!=NULL)
@@ -693,45 +626,30 @@ BOOLEAN iiEStart(char* example, procinfo *pi)
       currRing=NULL;
     }
   }
-#else /* USE_IILOCALRING */
-#endif /* USE_IILOCALRING */
-  if (NS_LRING != currRing)
-  {
-    if (NS_LRING!=NULL)
-    {
-      idhdl rh=procstack->cRingHdl;
-      if ((rh==NULL)||(IDRING(rh)!=NS_LRING))
-        rh=rFindHdl(NS_LRING,NULL);
-      rSetHdl(rh);
-    }
-    else
-    {
-      currRingHdl=NULL;
-      currRing=NULL;
-    }
-  }
-//#endif /* USE_IILOCALRING */
   procstack->pop();
   return err;
 }
 
-int staticdemo_mod_init(SModulFunctions*){ return 0; }
 
-#define SI_GET_BUILTIN_MOD_INIT(name) \
-  int name##_mod_init(SModulFunctions*); \
-  if (strcmp(libname, #name ".so") == 0) {  return name##_mod_init; }
+extern "C"
+{
+#  define SI_GET_BUILTIN_MOD_INIT0(name) int SI_MOD_INIT0(name)(SModulFunctions*);
+          SI_FOREACH_BUILTIN(SI_GET_BUILTIN_MOD_INIT0)
+#  undef  SI_GET_BUILTIN_MOD_INIT0
+};
 
 
 SModulFunc_t
-iiGetBuiltinModInit(char* libname)
+iiGetBuiltinModInit(const char* libname)
 {
-  SI_FOREACH_BUILTIN(SI_GET_BUILTIN_MOD_INIT)
+#  define SI_GET_BUILTIN_MOD_INIT(name) if (strcmp(libname, #name ".so") == 0){ return SI_MOD_INIT0(name); }
+          SI_FOREACH_BUILTIN(SI_GET_BUILTIN_MOD_INIT)
+#  undef  SI_GET_BUILTIN_MOD_INIT
 
   return NULL;
 }
 
 
-#undef SI_GET_BUILTIN_MOD_INIT
 
 
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
@@ -742,6 +660,9 @@ BOOLEAN iiTryLoadLib(leftv v, const char *id)
   char *libname = (char *)omAlloc(strlen(id)+5);
   const char *suffix[] = { "", ".lib", ".so", ".sl", NULL };
   int i = 0;
+  // FILE *fp;
+  // package pack;
+  // idhdl packhdl;
   lib_types LT;
   for(i=0; suffix[i] != NULL; i++)
   {
@@ -750,7 +671,9 @@ BOOLEAN iiTryLoadLib(leftv v, const char *id)
     if((LT = type_of_LIB(libname, libnamebuf)) > LT_NOTFOUND)
     {
       char *s=omStrDup(libname);
+      #ifdef HAVE_DYNAMIC_LOADING
       char libnamebuf[256];
+      #endif
 
       if (LT==LT_SINGULAR)
         LoadResult = iiLibCmd(s, FALSE, FALSE,TRUE);
@@ -780,41 +703,29 @@ BOOLEAN iiTryLoadLib(leftv v, const char *id)
 */
 BOOLEAN iiLocateLib(const char* lib, char* where)
 {
-  idhdl hl;
-
-  char *p;
-
-  hl = IDROOT->get("LIB", 0);
-  if (hl == NULL || (p=strstr(IDSTRING(hl), lib)) == NULL) return FALSE;
-  if ((p!=IDSTRING(hl)) && (*(p-1)!=',')) return FALSE;
-
-  if (strstr(IDSTRING(hl), ",") == NULL)
+  char *plib = iiConvName(lib);
+  idhdl pl = basePack->idroot->get(plib,0);
+  if( (pl!=NULL) && (IDTYP(pl)==PACKAGE_CMD) &&
+    (IDPACKAGE(pl)->language == LANG_SINGULAR))
   {
-    strcpy(where, IDSTRING(hl));
+    strncpy(where,IDPACKAGE(pl)->libname,127);
+    return TRUE;
   }
   else
-  {
-    char* tmp = omStrDup(IDSTRING(hl));
-    char* tok = strtok(tmp, ",");
-    do
-    {
-      if (strstr(tok, lib) != NULL) break;
-      tok = strtok(NULL, ",");
-    }
-    while (tok != NULL);
-    assume(tok != NULL);
-    strcpy(where, tok);
-    omFree(tmp);
-  }
-  return TRUE;
+    return FALSE;;
 }
 
 BOOLEAN iiLibCmd( char *newlib, BOOLEAN autoexport, BOOLEAN tellerror, BOOLEAN force )
 {
   char libnamebuf[128];
+  // procinfov pi;
+  // idhdl h;
   idhdl pl;
+  // idhdl hl;
+  // long pos = 0L;
   char *plib = iiConvName(newlib);
   FILE * fp = feFopen( newlib, "r", libnamebuf, tellerror );
+  // int lines = 1;
   BOOLEAN LoadResult = TRUE;
 
   if (fp==NULL)
@@ -885,6 +796,7 @@ static void iiRunInit(package p)
   {
     int save=yylineno;
     myynest++;
+    // procinfo *pi=(procinfo*)IDDATA(h);
     //PrintS("mod_init found\n");
     iiMake_proc(h,p,NULL);
     myynest--;
@@ -892,7 +804,7 @@ static void iiRunInit(package p)
   }
 }
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
-BOOLEAN iiLoadLIB(FILE *fp, const char *libnamebuf, char*newlib,
+BOOLEAN iiLoadLIB(FILE *fp, const char *libnamebuf, const char*newlib,
              idhdl pl, BOOLEAN autoexport, BOOLEAN tellerror)
 {
   extern FILE *yylpin;
@@ -911,7 +823,7 @@ BOOLEAN iiLoadLIB(FILE *fp, const char *libnamebuf, char*newlib,
   yylplex(newlib, libnamebuf, &lib_style, pl, autoexport);
   if(yylp_errno)
   {
-    Werror("Library %s: ERROR occured: in line %d, %d.", newlib, yylplineno,
+    Werror("Library %s: ERROR occurred: in line %d, %d.", newlib, yylplineno,
          current_pos(0));
     if(yylp_errno==YYLP_BAD_CHAR)
     {
@@ -921,7 +833,7 @@ BOOLEAN iiLoadLIB(FILE *fp, const char *libnamebuf, char*newlib,
     }
     else
       Werror(yylp_errlist[yylp_errno], yylplineno);
-    Werror("Cannot load library,... aborting.");
+    WerrorS("Cannot load library,... aborting.");
     reinit_yylp();
     fclose( yylpin );
     iiCleanProcs(IDROOT);
@@ -969,32 +881,15 @@ BOOLEAN iiLoadLIB(FILE *fp, const char *libnamebuf, char*newlib,
 
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 procinfo *iiInitSingularProcinfo(procinfov pi, const char *libname,
-              const char *procname, int line, long pos, BOOLEAN pstatic)
+              const char *procname, int, long pos, BOOLEAN pstatic)
 {
+  memset(pi,0,sizeof(*pi));
   pi->libname = omStrDup(libname);
-
-  if( strcmp(procname,"_init")==0)
-  {
-    pi->procname = iiConvName(libname);
-  }
-  else
-    pi->procname = omStrDup(procname);
+  pi->procname = omStrDup(procname);
   pi->language = LANG_SINGULAR;
   pi->ref = 1;
-  pi->pack = NULL;
   pi->is_static = pstatic;
   pi->data.s.proc_start = pos;
-  pi->data.s.def_end    = 0L;
-  pi->data.s.help_start = 0L;
-  pi->data.s.help_end   = 0L;
-  pi->data.s.body_start = 0L;
-  pi->data.s.body_end   = 0L;
-  pi->data.s.example_start = 0L;
-  pi->data.s.proc_lineno = line;
-  pi->data.s.body_lineno = 0;
-  pi->data.s.example_lineno = 0;
-  pi->data.s.body = NULL;
-  pi->data.s.help_chksum = 0;
   return(pi);
 }
 
@@ -1005,7 +900,7 @@ int iiAddCproc(const char *libname, const char *procname, BOOLEAN pstatic,
   procinfov pi;
   idhdl h;
 
-  #ifndef NDEBUG
+  #ifndef SING_NDEBUG
   int dummy;
   if (IsCmd(procname,dummy))
   {
@@ -1014,11 +909,25 @@ int iiAddCproc(const char *libname, const char *procname, BOOLEAN pstatic,
   }
   #endif
 
-  h = enterid(procname,0, PROC_CMD, &IDROOT, TRUE);
+  h=IDROOT->get(procname,0);
+  if ((h!=NULL)
+  && (IDTYP(h)==PROC_CMD))
+  {
+    pi = IDPROC(h);
+    if ((pi->language == LANG_SINGULAR)
+    &&(BVERBOSE(V_REDEFINE)))
+      Warn("extend `%s`",procname);
+  }
+  else
+  {
+    h = enterid(procname,0, PROC_CMD, &IDROOT, TRUE);
+  }
   if ( h!= NULL )
   {
     pi = IDPROC(h);
+    omfree(pi->libname);
     pi->libname = omStrDup(libname);
+    omfree(pi->procname);
     pi->procname = omStrDup(procname);
     pi->language = LANG_C;
     pi->ref = 1;
@@ -1028,7 +937,7 @@ int iiAddCproc(const char *libname, const char *procname, BOOLEAN pstatic,
   }
   else
   {
-    PrintS("iiAddCproc: failed.\n");
+    WarnS("iiAddCproc: failed.");
   }
   return(0);
 }
@@ -1046,17 +955,17 @@ int iiAddCprocTop(const char *libname, const char *procname, BOOLEAN pstatic,
 
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 #ifdef HAVE_DYNAMIC_LOADING
-BOOLEAN load_modules(char *newlib, char *fullname, BOOLEAN autoexport)
+BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport)
 {
 #ifdef HAVE_STATIC
   WerrorS("mod_init: static version can not load modules");
   return TRUE;
 #else
-  int iiAddCproc(const char *libname, const char *procname, BOOLEAN pstatic,
-                 BOOLEAN(*func)(leftv res, leftv v));
+/*
   typedef int (*fktn_t)(int(*iiAddCproc)(const char *libname, const char *procname,
                                BOOLEAN pstatic,
                                BOOLEAN(*func)(leftv res, leftv v)));
+*/
   SModulFunc_t fktn;
   idhdl pl;
   char *plib = iiConvName(newlib);
@@ -1076,30 +985,34 @@ BOOLEAN load_modules(char *newlib, char *fullname, BOOLEAN autoexport)
     Werror("'%s' is resered identifier\n", plib);
     goto load_modules_end;
   }
-  pl = IDROOT->get(plib,0);
-  if (pl==NULL)
+  pl = basePack->idroot->get(plib,0); /* packages only in top level
+                                        (see enterid) */
+  if ((pl!=NULL)
+  &&(IDTYP(pl)==PACKAGE_CMD))
   {
-    pl = enterid( plib,0, PACKAGE_CMD, &IDROOT,
-                  TRUE );
-    IDPACKAGE(pl)->language = LANG_C;
-    IDPACKAGE(pl)->libname=omStrDup(newlib);
+    if(IDPACKAGE(pl)->language==LANG_C)
+    {
+      if (BVERBOSE(V_LOAD_LIB)) Warn( "%s already loaded as package", newlib);
+      omFree(plib);
+      return FALSE;
+    }
   }
   else
   {
-    if(IDTYP(pl)!=PACKAGE_CMD)
-    {
-      Warn("not of type package.");
-      goto load_modules_end;
-    }
+    pl = enterid( plib,0, PACKAGE_CMD, &IDROOT, TRUE );
+    IDPACKAGE(pl)->libname=omStrDup(newlib);
   }
-  if (dynl_check_opened(FullName)) {
-    if (BVERBOSE(V_LOAD_LIB)) Warn( "%s already loaded", fullname);
+  IDPACKAGE(pl)->language = LANG_C;
+  if (dynl_check_opened(FullName))
+  {
+    if (BVERBOSE(V_LOAD_LIB)) Warn( "%s already loaded as C library", fullname);
     return FALSE;
   }
   if((IDPACKAGE(pl)->handle=dynl_open(FullName))==(void *)NULL)
   {
     Werror("dynl_open failed:%s", dynl_error());
     Werror("%s not found", newlib);
+    killhdl2(pl,&(basePack->idroot),NULL); // remove package
     goto load_modules_end;
   }
   else
@@ -1114,14 +1027,25 @@ BOOLEAN load_modules(char *newlib, char *fullname, BOOLEAN autoexport)
       sModulFunctions.iiArithAddCmd = iiArithAddCmd;
       if (autoexport) sModulFunctions.iiAddCproc = iiAddCprocTop;
       else            sModulFunctions.iiAddCproc = iiAddCproc;
-      (*fktn)(&sModulFunctions);
+      int ver=(*fktn)(&sModulFunctions);
+      if (ver==MAX_TOK)
+      {
+        if (BVERBOSE(V_LOAD_LIB)) Print( "// ** loaded %s\n", fullname);
+      }
+      else
+      {
+        Warn("loaded %s for a different version of Singular(expected MAX_TOK: %d, got %d)",fullname,MAX_TOK,ver);
+      }
+      currPack->loaded=1;
+      currPack=s;
+      RET=FALSE;
+    }
+    else
+    {
+      Werror("mod_init not found:: %s\nThis is probably not a dynamic module for Singular!\n", dynl_error());
+      killhdl2(pl,&(basePack->idroot),NULL); // remove package
     }
-    else Werror("mod_init: %s\n", dynl_error());
-    if (BVERBOSE(V_LOAD_LIB)) Print( "// ** loaded %s \n", fullname);
-    currPack->loaded=1;
-    currPack=s;
   }
-  RET=FALSE;
 
   load_modules_end:
   return RET;
@@ -1129,28 +1053,38 @@ BOOLEAN load_modules(char *newlib, char *fullname, BOOLEAN autoexport)
 }
 #endif /* HAVE_DYNAMIC_LOADING */
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
-BOOLEAN load_builtin(char *newlib, BOOLEAN autoexport, SModulFunc_t init)
+BOOLEAN load_builtin(const char *newlib, BOOLEAN autoexport, SModulFunc_t init)
 {
   int iiAddCproc(const char *libname, const char *procname, BOOLEAN pstatic,
                  BOOLEAN(*func)(leftv res, leftv v));
+/*
   typedef int (*fktn_t)(int(*iiAddCproc)(const char *libname, const char *procname,
                                BOOLEAN pstatic,
                                BOOLEAN(*func)(leftv res, leftv v)));
+*/
+  // SModulFunc_t fktn;
   idhdl pl;
   char *plib = iiConvName(newlib);
+  // BOOLEAN RET=TRUE;
+  // int token;
 
-  pl = IDROOT->get(plib,0);
-  if (pl!=NULL)
+  pl = basePack->idroot->get(plib,0); // search PACKAGE only in Top
+  if ((pl!=NULL)
+  &&(IDTYP(pl)==PACKAGE_CMD))
   {
-    if (BVERBOSE(V_LOAD_LIB)) Warn( "(builtin) %s already loaded", newlib);
-    omFree(plib);
-    return FALSE;
+    if(IDPACKAGE(pl)->language==LANG_C)
+    {
+      if (BVERBOSE(V_LOAD_LIB)) Warn( "(builtin) %s already loaded", newlib);
+      omFree(plib);
+      return FALSE;
+    }
+  }
+  else
+  {
+    pl = enterid( plib,0, PACKAGE_CMD, &IDROOT, TRUE );
+    IDPACKAGE(pl)->libname=omStrDup(newlib);
   }
-
-  pl = enterid( plib,0, PACKAGE_CMD, &IDROOT,
-                TRUE );
   IDPACKAGE(pl)->language = LANG_C;
-  IDPACKAGE(pl)->libname=omStrDup(newlib);
 
   IDPACKAGE(pl)->handle=(void *)NULL;
   SModulFunctions sModulFunctions;
@@ -1174,9 +1108,9 @@ BOOLEAN load_builtin(char *newlib, BOOLEAN autoexport, SModulFunc_t init)
 void module_help_main(const char *newlib,const char *help)
 {
   char *plib = iiConvName(newlib);
-  idhdl pl = basePack->idroot->get(plib,0); /* always start at Top level */
+  idhdl pl = basePack->idroot->get(plib,0);
   if ((pl==NULL)||(IDTYP(pl)!=PACKAGE_CMD))
-    Werror(">>%s<< is not a package",plib);
+    Werror(">>%s<< is not a package (trying to add package help)",plib);
   else
   {
     package s=currPack;
@@ -1191,7 +1125,7 @@ void module_help_proc(const char *newlib,const char *p, const char *help)
   char *plib = iiConvName(newlib);
   idhdl pl = basePack->idroot->get(plib,0);
   if ((pl==NULL)||(IDTYP(pl)!=PACKAGE_CMD))
-    Werror(">>%s<< is not a package",plib);
+    Werror(">>%s<< is not a package(trying to add help for %s)",plib,p);
   else
   {
     package s=currPack;
@@ -1206,16 +1140,14 @@ void module_help_proc(const char *newlib,const char *p, const char *help)
   }
 }
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
+
+#ifdef HAVE_DYNAMIC_LOADING
 // loads a dynamic module from the binary path and returns a named function
 // returns NULL, if something fails
 void* binary_module_function(const char* newlib, const char* funcname)
 {
   void* result = NULL;
 
-#ifdef HAVE_STATIC
-  WerrorS("static version can not load function from dynamic modules");
-#else
-#ifdef HAVE_DYNAMIC_LOADING
   const char* bin_dir = feGetResource('b');
   if (!bin_dir)  { return NULL; }
 
@@ -1230,12 +1162,10 @@ void* binary_module_function(const char* newlib, const char* funcname)
   }
   result = dynl_sym(openlib, funcname);
   if (!result) Werror("%s: %s\n", funcname, dynl_error());
-#endif /* HAVE_DYNAMIC_LOADING */
-#endif
 
   return result;
 }
-
+#endif
 
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 char mytoupper(char c)
@@ -1300,12 +1230,12 @@ void piShowProcList()
         Print( "%d %-15s  %20s ", proc->is_static ? 1 : 0, proc->libname,
                proc->procname);
       if(proc->language==LANG_SINGULAR)
-        Print("line %4d,%-5ld  %4d,%-5ld  %4d,%-5ld\n",
-              proc->data.s.proc_lineno, proc->data.s.proc_start,
+        Print("line %-5ld  %4d,%-5ld  %4d,%-5ld\n",
+              proc->data.s.proc_start,
               proc->data.s.body_lineno, proc->data.s.body_start,
               proc->data.s.example_lineno, proc->data.s.example_start);
       else if(proc->language==LANG_C)
-        Print("type: object\n");
+        PrintS("type: object\n");
     }
   }
 }
@@ -1325,7 +1255,7 @@ void piShowProcList()
 //}
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 #ifdef HAVE_LIBPARSER
-void libstack::push(char *p, char *libn)
+void libstack::push(const char */*p*/, char *libn)
 {
   libstackv lp;
   if( !iiGetLibStatus(libn))
@@ -1346,7 +1276,7 @@ void libstack::push(char *p, char *libn)
   }
 }
 
-libstackv libstack::pop(char *p)
+libstackv libstack::pop(const char */*p*/)
 {
   libstackv ls = this;
   //omFree((ADDRESS)ls->libname);
