@@ -5,36 +5,47 @@
  *
  * This file provides miscellaneous functionality.
  *
- * For more general information, see the documentation in
- * misc_ip.h.
- *
- * @author Frank Seelisch
+ * For more general information, see the documentation in misc_ip.h.
  *
  **/
 /*****************************************************************************/
 
 // include header files
+#define PLURAL_INTERNAL_DECLARATIONS 1
+
 #include <kernel/mod2.h>
-#include <Singular/lists.h>
-#include <kernel/longrat.h>
-#include <Singular/misc_ip.h>
-#include <Singular/feOpt.h>
-#include <Singular/silink.h>
+#include <misc/sirandom.h>
 
-void number2mpz(number n, mpz_t m)
-{
-  if (SR_HDL(n) & SR_INT) mpz_init_set_si(m, SR_TO_INT(n));     /* n fits in an int */
-  else             mpz_init_set(m, (mpz_ptr)n->z);
-}
+#include <reporter/si_signals.h>
+
+#include <factory/factory.h>
+
+#include <coeffs/si_gmp.h>
+#include <coeffs/coeffs.h>
+#include <coeffs/OPAE.h>
+#include <coeffs/OPAEQ.h>
+#include <coeffs/OPAEp.h>
+#include <coeffs/flintcf_Q.h>
+#include <coeffs/flintcf_Zn.h>
+
+#include <polys/ext_fields/algext.h>
+#include <polys/ext_fields/transext.h>
+#include <polys/nc/gb_hack.h>
+
+#ifdef HAVE_SIMPLEIPC
+#include <Singular/links/simpleipc.h>
+#endif
+
+#include "misc_ip.h"
+#include "ipid.h"
+#include "feOpt.h"
+#include "links/silink.h"
+#include "mod_lib.h"
+#include <Singular/distrib.h>
+
+static FORCE_INLINE void number2mpz(number n, mpz_t m){ number2mpz(n, coeffs_BIGINT, m); }
+static FORCE_INLINE number mpz2number(mpz_t m){ return mpz2number(m, coeffs_BIGINT); }
 
-number mpz2number(mpz_t m)
-{
-  number z = ALLOC_RNUMBER();
-  mpz_init_set(z->z, m);
-  mpz_init_set_ui(z->n, 1);
-  z->s = 3;
-  return z;
-}
 
 void setListEntry(lists L, int index, mpz_t n)
 { /* assumes n > 0 */
@@ -63,8 +74,7 @@ void setListEntry_ui(lists L, int index, unsigned long ui)
   }
   else
   {
-    number nn = nlRInit(ui);
-    mpz_set_ui(nn->z,ui);
+    number nn = n_Init(ui, coeffs_BIGINT);
     L->m[index].rtyp = BIGINT_CMD; L->m[index].data = (void*)nn;
   }
 }
@@ -106,7 +116,6 @@ static int factor_using_division (mpz_t t, unsigned int limit,lists primes, int
     setListEntry_ui(primes, index, 3);
     multiplicities[index++] = f;
   }
-
   f=0;
   loop
   {
@@ -330,9 +339,9 @@ lists primeFactorisation(const number n, const int pBound)
   int* multiplicities = (int*)omAlloc0(1000*sizeof(int));
   int positive=1;
 
-  if (!nlIsZero(n))
+  if (!n_IsZero(n, coeffs_BIGINT))
   {
-    if (!nlGreaterZero(n))
+    if (!n_GreaterZero(n, coeffs_BIGINT))
     {
       positive=-1;
       mpz_neg(nn,nn);
@@ -372,57 +381,35 @@ lists primeFactorisation(const number n, const int pBound)
   return L;
 }
 
-#include <string.h>
-#include <unistd.h>
-#include <stdio.h>
-#include <stddef.h>
-#include <stdlib.h>
-#include <time.h>
-
-#include <omalloc/mylimits.h>
 #include <omalloc/omalloc.h>
-#include <kernel/options.h>
-#include <kernel/febase.h>
-#include <Singular/cntrlc.h>
-#include <kernel/page.h>
-#include <Singular/ipid.h>
-#include <Singular/ipshell.h>
-#include <kernel/kstd1.h>
-#include <Singular/subexpr.h>
-#include <kernel/timer.h>
-#include <kernel/intvec.h>
-#include <kernel/ring.h>
-#include <kernel/p_Procs.h>
-/* Needed for debug Version of p_SetRingOfLeftv, Oliver */
-#ifdef PDEBUG
-#include <kernel/polys.h>
-#endif
-#include <Singular/version.h>
+#include <misc/mylimits.h>
+
+#include <misc/options.h>
+#include <misc/intvec.h>
+
+#include <polys/monomials/ring.h>
+#include <polys/templates/p_Procs.h>
+
+#include <kernel/GBEngine/kstd1.h>
+#include <kernel/oswrapper/timer.h>
+#include <resources/feResource.h>
+#include <kernel/oswrapper/feread.h>
+
+#include "subexpr.h"
+#include "cntrlc.h"
+#include "ipid.h"
+#include "ipshell.h"
+
+#include "fehelp.h"
 
-#include <Singular/static.h>
 #ifdef HAVE_STATIC
 #undef HAVE_DYN_RL
 #endif
 
-#define SI_DONT_HAVE_GLOBAL_VARS
-
 //#ifdef HAVE_LIBPARSER
 //#  include "libparse.h"
 //#endif /* HAVE_LIBPARSER */
 
-#ifdef HAVE_FACTORY
-#define SI_DONT_HAVE_GLOBAL_VARS
-#include <factory/factory.h>
-// libfac:
-  extern const char * libfac_version;
-  extern const char * libfac_date;
-#endif
-
-/* version strings */
-#include <kernel/si_gmp.h>
-#ifdef HAVE_MPSR
-#include <MP_Config.h>
-#endif
 
 /*2
 * the renice routine for very large jobs
@@ -441,6 +428,14 @@ lists primeFactorisation(const number n, const int pBound)
 *}
 */
 
+#include <string.h>
+#include <unistd.h>
+#include <stdio.h>
+#include <stddef.h>
+#include <stdlib.h>
+#include <time.h>
+
+
 void singular_example(char *str)
 {
   assume(str!=NULL);
@@ -518,14 +513,7 @@ void singular_example(char *str)
 }
 
 
-struct soptionStruct
-{
-  const char * name;
-  unsigned   setval;
-  unsigned   resetval;
-};
-
-struct soptionStruct optionStruct[]=
+const struct soptionStruct optionStruct[]=
 {
   {"prot",         Sy_bit(OPT_PROT),           ~Sy_bit(OPT_PROT)   },
   {"redSB",        Sy_bit(OPT_REDSB),          ~Sy_bit(OPT_REDSB)   },
@@ -555,7 +543,7 @@ struct soptionStruct optionStruct[]=
   {"ne",           0,                          0 }
 };
 
-struct soptionStruct verboseStruct[]=
+const struct soptionStruct verboseStruct[]=
 {
   {"mem",      Sy_bit(V_SHOW_MEM),  ~Sy_bit(V_SHOW_MEM)   },
   {"yacc",     Sy_bit(V_YACC),      ~Sy_bit(V_YACC)       },
@@ -578,8 +566,8 @@ struct soptionStruct verboseStruct[]=
   {"coefStrat",Sy_bit(V_COEFSTRAT), ~Sy_bit(V_COEFSTRAT)},
   {"qringNF",  Sy_bit(V_QRING),     ~Sy_bit(V_QRING)},
   {"warn",     Sy_bit(V_ALLWARN),   ~Sy_bit(V_ALLWARN)},
-  {"interedSyz",Sy_bit(V_INTERSECT_SYZ), ~Sy_bit(V_INTERSECT_SYZ)},
-  {"interedElim",Sy_bit(V_INTERSECT_ELIM), ~Sy_bit(V_INTERSECT_ELIM)},
+  {"intersectSyz",Sy_bit(V_INTERSECT_SYZ), ~Sy_bit(V_INTERSECT_SYZ)},
+  {"intersectElim",Sy_bit(V_INTERSECT_ELIM), ~Sy_bit(V_INTERSECT_ELIM)},
 /*special for "none" and also end marker for showOption:*/
   {"ne",         0,          0 }
 };
@@ -613,8 +601,8 @@ BOOLEAN setOption(leftv res, leftv v)
     if(strcmp(n,"get")==0)
     {
       intvec *w=new intvec(2);
-      (*w)[0]=test;
-      (*w)[1]=verbose;
+      (*w)[0]=si_opt_1;
+      (*w)[1]=si_opt_2;
       res->rtyp=INTVEC_CMD;
       res->data=(void *)w;
       goto okay;
@@ -626,16 +614,14 @@ BOOLEAN setOption(leftv res, leftv v)
       {
         v=v->next;
         intvec *w=(intvec*)v->Data();
-        test=(*w)[0];
-        verbose=(*w)[1];
+        si_opt_1=(*w)[0];
+        si_opt_2=(*w)[1];
 #if 0
         if (TEST_OPT_INTSTRATEGY && (currRing!=NULL)
         && rField_has_simple_inverse()
-#ifdef HAVE_RINGS
         && !rField_is_Ring(currRing)
-#endif
         ) {
-          test &=~Sy_bit(OPT_INTSTRATEGY);
+          si_opt_1 &=~Sy_bit(OPT_INTSTRATEGY);
         }
 #endif
         goto okay;
@@ -643,8 +629,8 @@ BOOLEAN setOption(leftv res, leftv v)
     }
     if(strcmp(n,"none")==0)
     {
-      test=0;
-      verbose=0;
+      si_opt_1=0;
+      si_opt_2=0;
       goto okay;
     }
     for (i=0; (i==0) || (optionStruct[i-1].setval!=0); i++)
@@ -653,19 +639,17 @@ BOOLEAN setOption(leftv res, leftv v)
       {
         if (optionStruct[i].setval & validOpts)
         {
-          test |= optionStruct[i].setval;
+          si_opt_1 |= optionStruct[i].setval;
           // optOldStd disables redthrough
           if (optionStruct[i].setval == Sy_bit(OPT_OLDSTD))
-            test &= ~Sy_bit(OPT_REDTHROUGH);
+            si_opt_1 &= ~Sy_bit(OPT_REDTHROUGH);
         }
         else
           Warn("cannot set option");
 #if 0
         if (TEST_OPT_INTSTRATEGY && (currRing!=NULL)
         && rField_has_simple_inverse()
-#ifdef HAVE_RINGS
         && !rField_is_Ring(currRing)
-#endif
         ) {
           test &=~Sy_bit(OPT_INTSTRATEGY);
         }
@@ -677,7 +661,7 @@ BOOLEAN setOption(leftv res, leftv v)
       {
         if (optionStruct[i].setval & validOpts)
         {
-          test &= optionStruct[i].resetval;
+          si_opt_1 &= optionStruct[i].resetval;
         }
         else
           Warn("cannot clear option");
@@ -688,7 +672,7 @@ BOOLEAN setOption(leftv res, leftv v)
     {
       if (strcmp(n,verboseStruct[i].name)==0)
       {
-        verbose |= verboseStruct[i].setval;
+        si_opt_2 |= verboseStruct[i].setval;
         #ifdef YYDEBUG
         #if YYDEBUG
         /*debugging the bison grammar --> grammar.cc*/
@@ -702,7 +686,7 @@ BOOLEAN setOption(leftv res, leftv v)
       else if ((strncmp(n,"no",2)==0)
       && (strcmp(n+2,verboseStruct[i].name)==0))
       {
-        verbose &= verboseStruct[i].resetval;
+        si_opt_2 &= verboseStruct[i].resetval;
         #ifdef YYDEBUG
         #if YYDEBUG
         /*debugging the bison grammar --> grammar.cc*/
@@ -717,14 +701,16 @@ BOOLEAN setOption(leftv res, leftv v)
     Werror("unknown option `%s`",n);
   okay:
     if (currRing != NULL)
-      currRing->options = test & TEST_RINGDEP_OPTS;
+      currRing->options = si_opt_1 & TEST_RINGDEP_OPTS;
     omFree((ADDRESS)n);
     v=v->next;
   } while (v!=NULL);
-  // set global variable to show memory usage
+
+   // set global variable to show memory usage
   extern int om_sing_opt_show_mem;
   if (BVERBOSE(V_SHOW_MEM)) om_sing_opt_show_mem = 1;
   else om_sing_opt_show_mem = 0;
+
   return FALSE;
 }
 
@@ -734,9 +720,9 @@ char * showOption()
   BITSET tmp;
 
   StringSetS("//options:");
-  if ((test!=0)||(verbose!=0))
+  if ((si_opt_1!=0)||(si_opt_2!=0))
   {
-    tmp=test;
+    tmp=si_opt_1;
     if(tmp)
     {
       for (i=0; optionStruct[i].setval!=0; i++)
@@ -752,7 +738,7 @@ char * showOption()
         if (tmp & Sy_bit(i)) StringAppend(" %d",i);
       }
     }
-    tmp=verbose;
+    tmp=si_opt_2;
     if (tmp)
     {
       for (i=0; verboseStruct[i].setval!=0; i++)
@@ -768,40 +754,56 @@ char * showOption()
         if (tmp & Sy_bit(i)) StringAppend(" %d",i+32);
       }
     }
-    return omStrDup(StringAppendS(""));
+    return StringEndS();
   }
-  else
-    return omStrDup(StringAppendS(" none"));
+  StringAppendS(" none");
+  return StringEndS();
 }
 
-char * versionString()
+/* version strings */
+#ifdef HAVE_FLINT
+extern "C"
 {
-  char* str = StringSetS("");
-  StringAppend("Singular for %s version %s (%d)  %s\nwith\n",
-               S_UNAME, S_VERSION1, SINGULAR_VERSION,
-               singular_date);
-  StringAppendS("\t");
-#ifdef HAVE_FACTORY
-              StringAppend("factory(%s),", factoryVersion);
-              StringAppend("libfac(%s,%s),\n\t",libfac_version,libfac_date);
+#ifndef __GMP_BITS_PER_MP_LIMB
+#define __GMP_BITS_PER_MP_LIMB GMP_LIMB_BITS
 #endif
-#if defined (__GNU_MP_VERSION) && defined (__GNU_MP_VERSION_MINOR)
-              StringAppend("GMP(%d.%d),",__GNU_MP_VERSION,__GNU_MP_VERSION_MINOR);
+#include <flint/flint.h>
+}
+#endif
+
+#ifndef MAKE_DISTRIBUTION
+const char *singular_date=__DATE__ " " __TIME__;
+#endif
+
+char * versionString(/*const bool bShowDetails = false*/ )
+{
+  StringSetS("");
+  StringAppend("Singular for %s version %s (%d, %d bit) %s #%s",
+               S_UNAME, VERSION, // SINGULAR_VERSION,
+               SINGULAR_VERSION, sizeof(void*)*8,
+#ifdef MAKE_DISTRIBUTION
+	       VERSION_DATE, GIT_VERSION);
 #else
-              StringAppendS("GMP(1.3),");
+	       singular_date, GIT_VERSION);
+#endif
+  StringAppendS("\nwith\n\t");
+
+#if defined(mpir_version)
+              StringAppend("MPIR(%s)~GMP(%s),", mpir_version, gmp_version);
+#elif defined(gmp_version)
+              // #if defined (__GNU_MP_VERSION) && defined (__GNU_MP_VERSION_MINOR)
+              //              StringAppend("GMP(%d.%d),",__GNU_MP_VERSION,__GNU_MP_VERSION_MINOR);
+              StringAppend("GMP(%s),", gmp_version);
 #endif
 #ifdef HAVE_NTL
 #include <NTL/version.h>
               StringAppend("NTL(%s),",NTL_VERSION);
 #endif
-#ifdef HAVE_MPSR
-              StringAppend("MP(%s),",MP_VERSION);
-#endif
-#if SIZEOF_VOIDP == 8
-              StringAppendS("64bit,");
-#else
-              StringAppendS("32bit,");
+
+#ifdef HAVE_FLINT
+              StringAppend("FLINT(%s),",version);
 #endif
+              StringAppend("factory(%s),\n\t", factoryVersion);
 #if defined(HAVE_DYN_RL)
               if (fe_fgets_stdin==fe_fgets_dummy)
                 StringAppendS("no input,");
@@ -829,9 +831,6 @@ char * versionString()
 #ifdef HAVE_PLURAL
               StringAppendS("Plural,");
 #endif
-#ifdef HAVE_FANS
-              StringAppendS("fan/cone,");
-#endif
 #ifdef HAVE_DBM
               StringAppendS("DBM,\n\t");
 #else
@@ -841,9 +840,6 @@ char * versionString()
               StringAppendS("dynamic modules,");
 #endif
               if (p_procs_dynamic) StringAppendS("dynamic p_Procs,");
-#ifdef TEST
-              StringAppendS("TESTs,");
-#endif
 #if YYDEBUG
               StringAppendS("YYDEBUG=1,");
 #endif
@@ -862,14 +858,23 @@ char * versionString()
 #ifdef OM_NDEBUG
               StringAppendS("OM_NDEBUG,");
 #endif
+#ifdef SING_NDEBUG
+              StringAppendS("SING_NDEBUG,");
+#endif
 #ifdef PDEBUG
               StringAppendS("PDEBUG,");
 #endif
 #ifdef KDEBUG
               StringAppendS("KDEBUG,");
 #endif
-#ifndef __OPTIMIZE__
-              StringAppendS("-g,");
+#ifdef __OPTIMIZE__
+              StringAppendS("CC:OPTIMIZE,");
+#endif
+#ifdef __OPTIMIZE_SIZE__
+              StringAppendS("CC:OPTIMIZE_SIZE,");
+#endif
+#ifdef __NO_INLINE__
+              StringAppendS("CC:NO_INLINE,");
 #endif
 #ifdef HAVE_EIGENVAL
               StringAppendS("eigenvalues,");
@@ -881,15 +886,26 @@ char * versionString()
               StringAppendS("ratGB,");
 #endif
               StringAppend("random=%d\n",siRandomStart);
-              StringAppend("\tCC=%s,\n\tCXX=%s"
+
+#define SI_SHOW_BUILTIN_MODULE(name) StringAppend(" %s", #name);
+              StringAppendS("built-in modules: {");
+              SI_FOREACH_BUILTIN(SI_SHOW_BUILTIN_MODULE)
+              StringAppendS("}\n");
+#undef SI_SHOW_BUILTIN_MODULE
+
+              StringAppend("AC_CONFIGURE_ARGS = %s,\n"
+                           "CC = %s,FLAGS : %s,\n"
+                           "CXX = %s,FLAGS : %s,\n"
+                           "DEFS : %s,CPPFLAGS : %s,\n"
+                           "LDFLAGS : %s,LIBS : %s "
 #ifdef __GNUC__
-              "(" __VERSION__ ")"
+              "(ver: " __VERSION__ ")"
 #endif
-              "\n",CC,CXX);
+              "\n",AC_CONFIGURE_ARGS, CC,CFLAGS, CXX,CXXFLAGS,  DEFS,CPPFLAGS,  LDFLAGS,LIBS);
               feStringAppendResources(0);
               feStringAppendBrowsers(0);
               StringAppendS("\n");
-              return str;
+              return StringEndS();
 }
 
 #ifdef PDEBUG
@@ -953,8 +969,7 @@ void listall(int showproc)
           else PrintS("   ");
           Print("::%s, typ %s level %d data %lx",
                  IDID(hh),Tok2Cmdname(IDTYP(hh)),IDLEV(hh),(long)IDDATA(hh));
-          if ((IDTYP(hh)==RING_CMD)
-          || (IDTYP(hh)==QRING_CMD))
+          if (IDTYP(hh)==RING_CMD)
             Print(" ref: %d\n",IDRING(hh)->ref);
           else
             PrintLn();
@@ -969,7 +984,6 @@ void listall(int showproc)
           IDID(hh),Tok2Cmdname(IDTYP(hh)),IDLEV(hh),(long)IDDATA(hh));
         else
         if ((IDTYP(hh)==RING_CMD)
-        || (IDTYP(hh)==QRING_CMD)
         || (IDTYP(hh)==PACKAGE_CMD))
         {
           Print("====== %s ==============\n",IDID(hh));
@@ -979,7 +993,7 @@ void listall(int showproc)
             if (showproc || (IDTYP(h2)!=PROC_CMD))
             {
               if ((IDDATA(h2)==(void *)currRing)
-              && ((IDTYP(h2)==RING_CMD)||(IDTYP(h2)==QRING_CMD)))
+              && (IDTYP(h2)==RING_CMD))
                 PrintS("(R)");
               else if (IDDATA(h2)==(void *)currPack) PrintS("(P)");
               else PrintS("   ");
@@ -996,7 +1010,7 @@ void listall(int showproc)
 }
 #endif
 
-#ifndef NDEBUG
+#ifndef SING_NDEBUG
 void checkall()
 {
       idhdl hh=basePack->idroot;
@@ -1004,7 +1018,10 @@ void checkall()
       {
         omCheckAddr(hh);
         omCheckAddr((ADDRESS)IDID(hh));
-        if (RingDependend(IDTYP(hh))) Print("%s typ %d in Top\n",IDID(hh),IDTYP(hh));
+        if (RingDependend(IDTYP(hh)))
+        {
+          Print("%s typ %d in Top (should be in ring)\n",IDID(hh),IDTYP(hh));
+        }
         hh=IDNEXT(hh);
       }
       hh=basePack->idroot;
@@ -1013,12 +1030,18 @@ void checkall()
         if (IDTYP(hh)==PACKAGE_CMD)
         {
           idhdl h2=IDPACKAGE(hh)->idroot;
-          while (h2!=NULL)
+          if (IDPACKAGE(hh)!=basePack)
           {
-            omCheckAddr(h2);
-            omCheckAddr((ADDRESS)IDID(h2));
-            if (RingDependend(IDTYP(h2))) Print("%s typ %d in %s\n",IDID(h2),IDTYP(h2),IDID(hh));
-            h2=IDNEXT(h2);
+            while (h2!=NULL)
+            {
+              omCheckAddr(h2);
+              omCheckAddr((ADDRESS)IDID(h2));
+              if (RingDependend(IDTYP(h2)))
+              {
+                Print("%s typ %d in %s (should be in ring)\n",IDID(h2),IDTYP(h2),IDID(hh));
+              }
+              h2=IDNEXT(h2);
+            }
           }
         }
         hh=IDNEXT(hh);
@@ -1033,92 +1056,108 @@ void checkall()
 extern "C"
 int singular_fstat(int fd, struct stat *buf)
 {
-  return fstat(fd,buf);
+  return si_fstat(fd,buf);
 }
 
 /*2
 * the global exit routine of Singular
 */
-#ifdef HAVE_MPSR
-void (*MP_Exit_Env_Ptr)()=NULL;
-#endif
-
 extern "C" {
+/* Note: We cannot use a mutex here because mutexes are not async-safe, but
+ * m2_end is called by sig_term_hdl(). Anyway, the race condition in the first
+ * few lines of m2_end() should not matter.
+ */
+volatile BOOLEAN m2_end_called = FALSE;
 
 void m2_end(int i)
 {
-  fe_reset_input_mode();
-  #ifdef PAGE_TEST
-  mmEndStat();
-  #endif
-  fe_reset_input_mode();
-  if (ssiToBeClosed_inactive)
+  if (!m2_end_called)
   {
-    link_list hh=ssiToBeClosed;
-    while(hh!=NULL)
+    extern FILE* File_Profiling;
+    if (File_Profiling!=NULL) { fclose(File_Profiling); File_Profiling=NULL; }
+    m2_end_called = TRUE;
+#ifdef HAVE_SIMPLEIPC
+    for (int j = SIPC_MAX_SEMAPHORES-1; j >= 0; j--)
     {
-      //Print("close %s\n",hh->l->name);
-      slPrepClose(hh->l);
-      hh=(link_list)hh->next;
-    }
-    ssiToBeClosed_inactive=FALSE;
-
-    idhdl h = currPack->idroot;
-    while(h != NULL)
-    {
-      if(IDTYP(h) == LINK_CMD)
+      if (semaphore[j] != NULL)
       {
-        idhdl hh=h->next;
-        //Print("kill %s\n",IDID(h));
-        killhdl(h, currPack);
-        h = hh;
+        while (sem_acquired[j] > 0)
+        {
+          sem_post(semaphore[j]);
+          sem_acquired[j]--;
+        }
+      }
+    }
+#endif   // HAVE_SIMPLEIPC
+    fe_reset_input_mode();
+    monitor(NULL,0);
+#ifdef PAGE_TEST
+    mmEndStat();
+#endif
+    fe_reset_input_mode();
+    if (ssiToBeClosed_inactive)
+    {
+      link_list hh=ssiToBeClosed;
+      while(hh!=NULL)
+      {
+        //Print("close %s\n",hh->l->name);
+        slPrepClose(hh->l);
+        hh=(link_list)hh->next;
+      }
+      ssiToBeClosed_inactive=FALSE;
+
+      idhdl h = currPack->idroot;
+      while(h != NULL)
+      {
+        if(IDTYP(h) == LINK_CMD)
+        {
+          idhdl hh=h->next;
+          //Print("kill %s\n",IDID(h));
+          killhdl(h, currPack);
+          h = hh;
+        }
+        else
+        {
+          h = h->next;
+        }
+      }
+      hh=ssiToBeClosed;
+      while(hh!=NULL)
+      {
+        //Print("close %s\n",hh->l->name);
+        slClose(hh->l);
+        hh=ssiToBeClosed;
+      }
+    }
+    if (!singular_in_batchmode)
+    {
+      if (i<=0)
+      {
+        if (TEST_V_QUIET)
+        {
+          if (i==0)
+            printf("Auf Wiedersehen.\n");
+          else
+            printf("\n$Bye.\n");
+        }
+        //#ifdef sun
+        //  #ifndef __svr4__
+        //    _cleanup();
+        //    _exit(0);
+        //  #endif
+        //#endif
+        i=0;
       }
       else
       {
-        h = h->next;
+        printf("\nhalt %d\n",i);
       }
     }
-    hh=ssiToBeClosed;
-    while(hh!=NULL)
-    {
-      //Print("close %s\n",hh->l->name);
-      slClose(hh->l);
-      hh=ssiToBeClosed;
-    }
+    exit(i);
   }
-  if (!singular_in_batchmode)
-  {
-    if (i<=0)
-    {
-      if (TEST_V_QUIET)
-      {
-        if (i==0)
-          printf("Auf Wiedersehen.\n");
-        else
-          printf("\n$Bye.\n");
-      }
-      //#ifdef sun
-      //  #ifndef __svr4__
-      //    _cleanup();
-      //    _exit(0);
-      //  #endif
-      //#endif
-      i=0;
-    }
-    else
-    {
-      printf("\nhalt %d\n",i);
-    }
-  }
-  #ifdef HAVE_MPSR
-  if (MP_Exit_Env_Ptr!=NULL) (*MP_Exit_Env_Ptr)();
-  #endif
-  exit(i);
 }
 }
 
-const char *singular_date=__DATE__ " " __TIME__;
-
 extern "C"
 {
   void omSingOutOfMemoryFunc()
@@ -1131,16 +1170,51 @@ extern "C"
   }
 }
 
+#ifdef SINGULAR_4_1
+static n_coeffType n_pAE=n_unknown;
+static BOOLEAN ii_pAE_init(leftv res,leftv a)
+{
+  if (a->Typ()!=INT_CMD)
+  {
+    WerrorS("`int` expected");
+    return TRUE;
+  }
+  else
+  {
+    res->rtyp=CRING_CMD;
+    res->data=(void*)nInitChar(n_pAE,(void*)a->Data());
+    return FALSE;
+  }
+}
+#ifdef HAVE_FLINT
+static n_coeffType n_FlintZn=n_unknown;
+static BOOLEAN ii_FlintZn_init(leftv res,leftv a)
+{
+  if ((a->Typ()!=INT_CMD)
+  ||(a->next==NULL)
+  ||(a->next->Typ()!=STRING_CMD))
+  {
+    WerrorS("`int`i,`string` expected");
+    return TRUE;
+  }
+  else
+  {
+    flintZn_struct p;
+    p.ch=(int)(long)a->Data();
+    p.name=(char*)a->next->Data();
+    res->rtyp=CRING_CMD;
+    res->data=(void*)nInitChar(n_FlintZn,(void*)&p);
+    return FALSE;
+  }
+}
+#endif
+#endif
 /*2
 * initialize components of Singular
 */
 void siInit(char *name)
 {
-#ifdef HAVE_FACTORY
 // factory default settings: -----------------------------------------------
-  On(SW_USE_NTL);
-  On(SW_USE_NTL_GCD_0); // On -> seg11 in Old/algnorm, Old/factor...
-  On(SW_USE_NTL_GCD_P); // On -> cyle in Short/brnoeth_s: fixed
   On(SW_USE_EZGCD);
   On(SW_USE_CHINREM_GCD);
   //On(SW_USE_FF_MOD_GCD);
@@ -1148,28 +1222,28 @@ void siInit(char *name)
   On(SW_USE_QGCD);
   Off(SW_USE_NTL_SORT); // may be changed by an command line option
   factoryError=WerrorS;
-#endif
 
 // memory initialization: -----------------------------------------------
     om_Opts.OutOfMemoryFunc = omSingOutOfMemoryFunc;
 #ifndef OM_NDEBUG
 #ifndef __OPTIMIZE__
     om_Opts.ErrorHook = dErrorBreak;
+#else
+    om_Opts.Keep = 0; /* !OM_NDEBUG, __OPTIMIZE__*/
 #endif
+#else
+    om_Opts.Keep = 0; /* OM_NDEBUG */
 #endif
     omInitInfo();
-#ifdef OM_SING_KEEP
-    om_Opts.Keep = OM_SING_KEEP;
-#endif
 
+// options ---------------------------------------------------------------
+  si_opt_1=0;
 // interpreter tables etc.: -----------------------------------------------
-#ifdef INIT_BUG
-  jjInitTab1();
-#endif
   memset(&sLastPrinted,0,sizeof(sleftv));
   sLastPrinted.rtyp=NONE;
-  extern int iiInitArithmetic();
-  iiInitArithmetic();
+
+  extern int iiInitArithmetic(); iiInitArithmetic(); // iparith.cc
+
   basePack=(package)omAlloc0(sizeof(*basePack));
   currPack=basePack;
   idhdl h;
@@ -1179,18 +1253,38 @@ void siInit(char *name)
   currPackHdl=h;
   basePackHdl=h;
 
+  coeffs_BIGINT = nInitChar(n_Q,(void*)1);
+
+#if 1
+   // def HAVE_POLYEXTENSIONS
+  if(TRUE)
+  {
+    n_coeffType type;
+    #ifdef SINGULAR_4_1
+    type = nRegister(n_polyExt, naInitChar);
+    assume(type == n_polyExt);
+    #endif
+
+    type = nRegister(n_algExt, naInitChar);
+    assume(type == n_algExt);
+
+    type = nRegister(n_transExt, ntInitChar);
+    assume(type == n_transExt);
+
+    (void)type;
+  }
+#endif
+
 // random generator: -----------------------------------------------
   int t=initTimer();
   if (t==0) t=1;
   initRTimer();
   siSeed=t;
-#ifdef HAVE_FACTORY
   factoryseed(t);
-#endif
   siRandomStart=t;
   feOptSpec[FE_OPT_RANDOM].value = (void*) ((long)siRandomStart);
 
-// resource table: ----------------------------------------------------
+// ressource table: ----------------------------------------------------
   // Don't worry: ifdef OM_NDEBUG, then all these calls are undef'ed
   // hack such that all shared' libs in the bindir are loaded correctly
   feInitResources(name);
@@ -1208,23 +1302,68 @@ void siInit(char *name)
   #endif
   feSetOptValue(FE_OPT_CPUS, cpus);
 
+#ifdef SINGULAR_4_1
+// default coeffs
+  {
+    idhdl h;
+    h=enterid(omStrDup("QQ"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
+    IDDATA(h)=(char*)nInitChar(n_Q,NULL);
+    h=enterid(omStrDup("ZZ"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
+    IDDATA(h)=(char*)nInitChar(n_Z,NULL);
+    //h=enterid(omStrDup("RR"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
+    //IDDATA(h)=(char*)nInitChar(n_R,NULL);
+    //h=enterid(omStrDup("CC"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
+    //IDDATA(h)=(char*)nInitChar(n_long_C,NULL);
+    n_coeffType t=nRegister(n_unknown,n_AEInitChar);
+    if (t!=n_unknown)
+    {
+      h=enterid(omStrDup("AE"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
+      IDDATA(h)=(char*)nInitChar(t,NULL);
+    }
+    t=nRegister(n_unknown,n_QAEInitChar);
+    if (t!=n_unknown)
+    {
+      h=enterid(omStrDup("QAE"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
+      IDDATA(h)=(char*)nInitChar(t,NULL);
+    }
+    n_pAE=nRegister(n_unknown,n_pAEInitChar);
+    if (n_pAE!=n_unknown)
+    {
+      iiAddCproc("kernel","pAE",FALSE,ii_pAE_init);
+    }
+    #ifdef HAVE_FLINT
+    t=nRegister(n_unknown,flintQ_InitChar);
+    if (t!=n_unknown)
+    {
+      h=enterid(omStrDup("flint_poly_Q"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
+      IDDATA(h)=(char*)nInitChar(t,NULL);
+    }
+    n_FlintZn=nRegister(n_unknown,flintZn_InitChar);
+    if (n_FlintZn!=n_unknown)
+    {
+      iiAddCproc("kernel","flintZ",FALSE,ii_FlintZn_init);
+    }
+    #endif
+  }
+#endif
+// setting routines for PLURAL QRINGS:
+// allowing to use libpolys without libSingular(kStd)
+#ifdef HAVE_PLURAL
+  nc_NF=k_NF;
+  gnc_gr_bba=k_gnc_gr_bba;
+  gnc_gr_mora=k_gnc_gr_mora;
+  sca_bba=k_sca_bba;
+  sca_mora=k_sca_mora;
+  sca_gr_bba=k_sca_gr_bba;
+#endif
 // loading standard.lib -----------------------------------------------
   if (! feOptValue(FE_OPT_NO_STDLIB))
   {
-    int vv=verbose;
-    verbose &= ~Sy_bit(V_LOAD_LIB);
+    BITSET save1,save2;
+    SI_SAVE_OPT(save1,save2);
+    si_opt_2 &= ~Sy_bit(V_LOAD_LIB);
     iiLibCmd(omStrDup("standard.lib"), TRUE,TRUE,TRUE);
-    verbose=vv;
+    SI_RESTORE_OPT(save1,save2);
   }
   errorreported = 0;
 }
-
-#ifdef LIBSINGULAR
-#ifdef HAVE_FACTORY
-// the init routines of factory need mmInit
-int mmInit( void )
-{
-  return 1;
-}
-#endif
-#endif
