@@ -5,39 +5,47 @@
 * ABSTRACT: handling of leftv
 */
 
+#include <kernel/mod2.h>
+
+#include <omalloc/omalloc.h>
+
+#include <misc/intvec.h>
+#include <misc/options.h>
+
+#include <coeffs/numbers.h>
+#include <coeffs/bigintmat.h>
+
+#include <coeffs/ffields.h> // nfShowMipo // minpoly printing...
+
+#include <polys/monomials/maps.h>
+#include <polys/matpol.h>
+#include <polys/monomials/ring.h>
+
+// #include <coeffs/longrat.h>
+
+#include <kernel/polys.h>
+#include <kernel/ideals.h>
+#include <kernel/GBEngine/kstd1.h>
+#include <kernel/GBEngine/syz.h>
+#include <kernel/oswrapper/timer.h>
+
+#include <Singular/tok.h>
+#include <Singular/ipid.h>
+#include <Singular/ipshell.h>
+#include <Singular/lists.h>
+#include <Singular/attrib.h>
+#include <Singular/links/silink.h>
+#include <Singular/attrib.h>
+#include <Singular/subexpr.h>
+#include <Singular/blackbox.h>
+#include <Singular/number2.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
 
-#include <kernel/mod2.h>
-#include <kernel/intvec.h>
-#include <kernel/bigintmat.h>
-#include <Singular/tok.h>
-#include <kernel/options.h>
-#include <Singular/ipid.h>
-#include <omalloc/omalloc.h>
-#include <kernel/febase.h>
-#include <kernel/polys.h>
-#include <kernel/ideals.h>
-#include <kernel/maps.h>
-#include <kernel/matpol.h>
-#include <kernel/kstd1.h>
-#include <kernel/timer.h>
-#include <kernel/ring.h>
-#include <kernel/ffields.h>
-#include <kernel/numbers.h>
-#include <kernel/longrat.h>
-#include <Singular/ipshell.h>
-#include <Singular/lists.h>
-#include <Singular/attrib.h>
-#include <Singular/silink.h>
-#include <kernel/syz.h>
-#include <Singular/attrib.h>
-#include <Singular/subexpr.h>
-#include <Singular/blackbox.h>
-
 omBin sSubexpr_bin = omGetSpecBin(sizeof(_ssubexpr));
 omBin sleftv_bin = omGetSpecBin(sizeof(sleftv));
 omBin procinfo_bin = omGetSpecBin(sizeof(procinfo));
@@ -94,11 +102,26 @@ void sleftv::Print(leftv store, int spaces)
     char *s;
     void *d=Data();
     if (errorreported) return;
-    if ((store!=NULL)&&(store!=this))
-      store->CleanUp();
 
     switch (t /*=Typ()*/)
       {
+#ifdef SINGULAR_4_1
+        case CRING_CMD:
+          crPrint((coeffs)d);
+          break;
+#endif
+#ifdef SINGULAR_4_2
+        case CNUMBER_CMD:
+          n2Print((number2)d);
+          break;
+        case CPOLY_CMD:
+          p2Print((poly2)d);
+          break;
+        case CMATRIX_CMD: // like BIGINTMAT
+#endif
+        case BIGINTMAT_CMD:
+          ((bigintmat *)d)->pprint(80);
+          break;
         case UNKNOWN:
         case DEF_CMD:
           PrintNSpaces(spaces);
@@ -108,26 +131,26 @@ void sleftv::Print(leftv store, int spaces)
           PrintNSpaces(spaces);
           paPrint(n,(package)d);
           break;
+	case LIB_CMD:
         case NONE:
           return;
         case INTVEC_CMD:
         case INTMAT_CMD:
           ((intvec *)d)->show(t,spaces);
           break;
-        case BIGINTMAT_CMD:
-          ((bigintmat *)d)->pprint(80);
-          break;
         case RING_CMD:
-        case QRING_CMD:
+        {
           PrintNSpaces(spaces);
-          rWrite((ring)d);
+          const ring r = (const ring)d;
+          rWrite(r, currRing == r);
           break;
+        }
         case MATRIX_CMD:
-          iiWriteMatrix((matrix)d,n,2,spaces);
+          iiWriteMatrix((matrix)d,n,2, currRing, spaces);
           break;
         case MODUL_CMD:
         case IDEAL_CMD:
-          if ((TEST_V_QRING)  &&(currQuotient!=NULL)
+          if ((TEST_V_QRING)  &&(currRing->qideal!=NULL)
           &&(!hasFlag(this,FLAG_QRING)))
           {
             jjNormalizeQRingId(this);
@@ -135,15 +158,29 @@ void sleftv::Print(leftv store, int spaces)
           }
           // no break:
         case MAP_CMD:
-          iiWriteMatrix((matrix)d,n,1,spaces);
+          iiWriteMatrix((matrix)d,n,1, currRing, spaces);
           break;
         case POLY_CMD:
         case VECTOR_CMD:
-          if ((TEST_V_QRING)  &&(currQuotient!=NULL)
+          if ((e==NULL)
+          && (TEST_V_QRING)
+          &&(currRing->qideal!=NULL)
           &&(!hasFlag(this,FLAG_QRING)))
           {
-            jjNormalizeQRingP(this);
-            d=Data();
+            setFlag(this,FLAG_QRING);
+            poly p=(poly)d;
+            jjNormalizeQRingP(p);
+            if (p!=(poly)d)
+            {
+              d=(void*)p;
+              if ((rtyp==POLY_CMD)||(rtyp==VECTOR_CMD)) data=d;
+              else if (rtyp==IDHDL)
+              {
+                idhdl h=(idhdl)data;
+                IDPOLY(h)=p;
+                setFlag(h,FLAG_QRING);
+              }
+            }
           }
           PrintNSpaces(spaces);
           pWrite0((poly)d);
@@ -151,7 +188,7 @@ void sleftv::Print(leftv store, int spaces)
         case RESOLUTION_CMD:
         {
           syStrategy tmp=(syStrategy)d;
-          syPrint(tmp);
+          syPrint(tmp,IDID(currRingHdl));
           break;
         }
         case STRING_CMD:
@@ -183,15 +220,6 @@ void sleftv::Print(leftv store, int spaces)
            //   piProcinfo(pi, "ref"));
            break;
          }
-       case POINTER_CMD:
-         { package pack = (package)d;
-         PrintNSpaces(spaces);
-         PrintS("// PointerTest\n");
-         PrintNSpaces(spaces);
-         ::Print("// %s\n",IDID(pack->idroot));
-         //::Print(((char *)(pack->idroot)->data), spaces);
-         break;
-         }
        case LINK_CMD:
           {
             si_link l=(si_link)d;
@@ -209,7 +237,6 @@ void sleftv::Print(leftv store, int spaces)
             ::Print("// write: %s", slStatus(l, "write"));
           break;
           }
-        case NUMBER_CMD:
         case BIGINT_CMD:
           s=String(d);
           if (s==NULL) return;
@@ -217,6 +244,22 @@ void sleftv::Print(leftv store, int spaces)
           PrintS(s);
           omFree((ADDRESS)s);
           break;
+        case NUMBER_CMD:
+          {
+            number n=(number)d;
+            nNormalize(n);
+            if ((number)d !=n)
+            {
+              d=n;
+              if (rtyp==IDHDL) IDNUMBER(((idhdl)data))=n;
+              else if(rtyp==NUMBER_CMD) data=(void*)n;
+            }
+            s=String(d);
+            if (s==NULL) return;
+            PrintS(s);
+            omFree((ADDRESS)s);
+            break;
+          }
         case LIST_CMD:
         {
           lists l=(lists)d;
@@ -252,6 +295,8 @@ void sleftv::Print(leftv store, int spaces)
           else
           ::Print("Print:unknown type %s(%d)", Tok2Cmdname(t),t);
       } /* end switch: (Typ()) */
+    if ((store!=NULL)&&(store!=this))
+      store->CleanUp();
   }
   if (next!=NULL)
   {
@@ -271,7 +316,6 @@ void sleftv::Print(leftv store, int spaces)
     && (store!=this))
     {
       if((t/*Typ()*/!=LINK_CMD)
-      && (t/*Typ()*/!=POINTER_CMD)
       && (t/*Typ()*/!=PACKAGE_CMD)
       && (t/*Typ()*/!=DEF_CMD)
       )
@@ -290,48 +334,47 @@ void sleftv::Print(leftv store, int spaces)
 
 void sleftv::CleanUp(ring r)
 {
-  if ((name!=NULL) && (name!=sNoName) && (rtyp!=IDHDL) && (rtyp!=ALIAS_CMD))
+  if (rtyp!=IDHDL)
   {
-    //::Print("free %x (%s)\n",name,name);
-    //omFreeBinAddr((ADDRESS)name);
-    omFree((ADDRESS)name); /* here, a name (monomial) may really be larger
-                              then 1000 chars => not a BinAddr */
-  }
-  //name=NULL;
-  //flag=0;
-  if (data!=NULL)
-  {
-     if (rtyp==IDHDL) attribute=NULL; // is only a pointer to attribute of id
-     else s_internalDelete(rtyp,data,r);
-    //data=NULL; // will be done by Init() at the end
-  }
-  if (attribute!=NULL)
-  {
-    switch (rtyp)
+    if ((name!=NULL) && (name!=sNoName) && (rtyp!=ALIAS_CMD))
     {
-      case POINTER_CMD:
-      case PACKAGE_CMD:
-      case IDHDL:
-      case ANY_TYPE:
-      case VECHO:
-      case VPRINTLEVEL:
-      case VCOLMAX:
-      case VTIMER:
-      case VRTIMER:
-      case VOICE:
-      case VMAXDEG:
-      case VMAXMULT:
-      case TRACE:
-      case VSHORTOUT:
-      case VNOETHER:
-      case VMINPOLY:
-      case LIB_CMD:
-      case 0:
-        //attribute=NULL; // will be done by Init() at the end
-        break;
-      default:
+      //::Print("free %x (%s)\n",name,name);
+      omFree((ADDRESS)name);
+    }
+    //name=NULL;
+    //flag=0;
+    if (data!=NULL)
+    {
+      //if (rtyp==IDHDL) attribute=NULL; // is only a pointer to attribute of id
+      s_internalDelete(rtyp,data,r);
+      //data=NULL; // will be done by Init() at the end
+    }
+    if (attribute!=NULL)
+    {
+      switch (rtyp)
       {
-        attribute->killAll(r);
+        case PACKAGE_CMD:
+        //case IDHDL:
+        case ANY_TYPE:
+        case VECHO:
+        case VPRINTLEVEL:
+        case VCOLMAX:
+        case VTIMER:
+        case VRTIMER:
+        case VOICE:
+        case VMAXDEG:
+        case VMAXMULT:
+        case TRACE:
+        case VSHORTOUT:
+        case VNOETHER:
+        case VMINPOLY:
+        case 0:
+          //attribute=NULL; // will be done by Init() at the end
+          break;
+        default:
+        {
+          attribute->killAll(r);
+        }
       }
     }
   }
@@ -362,7 +405,7 @@ void sleftv::CleanUp(ring r)
 BOOLEAN sleftv::RingDependend()
 {
   int rt=Typ();
-  if(::RingDependend(rt) && (rt!=QRING_CMD))
+  if(::RingDependend(rt))
     return TRUE;
   if (rt==LIST_CMD)
     return lRingDependend((lists)Data());
@@ -375,20 +418,33 @@ static inline void * s_internalCopy(const int t,  void *d)
 {
   switch (t)
   {
+#ifdef SINGULAR_4_1
+    case CRING_CMD:
+      {
+        coeffs cf=(coeffs)d;
+        cf->ref++;
+        return (void*)d;
+      }
+#endif
+#ifdef SINGULAR_4_2
+    case CNUMBER_CMD:
+      return (void*)n2Copy((number2)d);
+    case CPOLY_CMD:
+      return (void*)p2Copy((poly2)d);
+    case CMATRIX_CMD: // like BIGINTMAT
+#endif
+    case BIGINTMAT_CMD:
+      return (void*)bimCopy((bigintmat *)d);
     case INTVEC_CMD:
     case INTMAT_CMD:
       return (void *)ivCopy((intvec *)d);
-    case BIGINTMAT_CMD:
-      return (void*)bimCopy((bigintmat *)d);
     case MATRIX_CMD:
-      return (void *)mpCopy((matrix)d);
+      return (void *)mp_Copy((matrix)d, currRing);
     case IDEAL_CMD:
     case MODUL_CMD:
       return  (void *)idCopy((ideal)d);
     case STRING_CMD:
         return (void *)omStrDup((char *)d);
-    case POINTER_CMD:
-      return d;
     case PACKAGE_CMD:
       return  (void *)paCopy((package) d);
     case PROC_CMD:
@@ -401,19 +457,21 @@ static inline void * s_internalCopy(const int t,  void *d)
     case NUMBER_CMD:
       return  (void *)nCopy((number)d);
     case BIGINT_CMD:
-      return  (void *)nlCopy((number)d);
+      return  (void *)n_Copy((number)d, coeffs_BIGINT);
     case MAP_CMD:
-      return  (void *)maCopy((map)d);
+      return  (void *)maCopy((map)d, currRing);
     case LIST_CMD:
       return  (void *)lCopy((lists)d);
     case LINK_CMD:
       return (void *)slCopy((si_link) d);
     case RING_CMD:
-    case QRING_CMD:
       {
         ring r=(ring)d;
-        if (r!=NULL) r->ref++;
-        //Print("+  ring %d, ref %d\n",r,r->ref);
+        if (r!=NULL)
+        {
+          r->ref++;
+          //Print("s_internalCopy:+  ring %d, ref %d\n",r,r->ref);
+        }
         return d;
       }
     case RESOLUTION_CMD:
@@ -432,7 +490,7 @@ static inline void * s_internalCopy(const int t,  void *d)
         return NULL;
       }
       else
-        Warn("s_internalCopy: cannot copy type %s(%d)",
+      Warn("s_internalCopy: cannot copy type %s(%d)",
             Tok2Cmdname(t),t);
     }
   }
@@ -444,6 +502,42 @@ void s_internalDelete(const int t,  void *d, const ring r)
   assume(d!=NULL);
   switch (t)
   {
+#ifdef SINGULAR_4_1
+    case CRING_CMD:
+      {
+        coeffs cf=(coeffs)d;
+        if ((cf->ref<=1)&&
+        ((cf->type <=n_long_R)
+          ||((cf->type >=n_long_C)&&(cf->type <=n_CF))))
+        {
+          Werror("cannot kill %s",nCoeffName(cf));
+        }
+        else
+          nKillChar((coeffs)d);
+        break;
+      }
+#endif
+#ifdef SINGULAR_4_2
+    case CNUMBER_CMD:
+      {
+        number2 n=(number2)d;
+        n2Delete(n);
+        break;
+      }
+    case CPOLY_CMD:
+      {
+        poly2 n=(poly2)d;
+        p2Delete(n);
+        break;
+      }
+    case CMATRIX_CMD: //like BIGINTMAT
+#endif
+    case BIGINTMAT_CMD:
+    {
+      bigintmat *v=(bigintmat*)d;
+      delete v;
+      break;
+    }
     case INTVEC_CMD:
     case INTMAT_CMD:
     {
@@ -451,12 +545,6 @@ void s_internalDelete(const int t,  void *d, const ring r)
       delete v;
       break;
     }
-    case BIGINTMAT_CMD:
-    {
-      bigintmat *v=(bigintmat*)d;
-      delete v;
-      break;
-    }
     case MAP_CMD:
     {
       map m=(map)d;
@@ -475,8 +563,6 @@ void s_internalDelete(const int t,  void *d, const ring r)
     case STRING_CMD:
       omFree(d);
       break;
-    //case POINTER_CMD:
-    //  return d;
     //case PACKAGE_CMD:
     //  return  (void *)paCopy((package) d);
     case PROC_CMD:
@@ -492,13 +578,13 @@ void s_internalDelete(const int t,  void *d, const ring r)
     case NUMBER_CMD:
     {
       number n=(number)d;
-      n_Delete(&n,r);
+      n_Delete(&n,r->cf);
       break;
     }
     case BIGINT_CMD:
     {
       number n=(number)d;
-      nlDelete(&n,NULL);
+      n_Delete(&n,coeffs_BIGINT);
       break;
     }
     case LIST_CMD:
@@ -514,7 +600,6 @@ void s_internalDelete(const int t,  void *d, const ring r)
       break;
     }
     case RING_CMD:
-    case QRING_CMD:
     {
       ring R=(ring)d;
       if ((R!=currRing)||(R->ref>=0))
@@ -543,7 +628,6 @@ void s_internalDelete(const int t,  void *d, const ring r)
     case INT_CMD:
     case DEF_CMD:
     case ALIAS_CMD:
-    case POINTER_CMD:
     case PACKAGE_CMD:
     case IDHDL:
     case NONE:
@@ -560,7 +644,6 @@ void s_internalDelete(const int t,  void *d, const ring r)
     case VSHORTOUT:
     case VNOETHER:
     case VMINPOLY:
-    case LIB_CMD:
     case 0: /* type in error case */
       break; /* error recovery: do nothing */
     //case COMMAND:
@@ -586,7 +669,10 @@ void * slInternalCopy(leftv source, const int t, void *d, Subexpr e)
   {
       if ((e==NULL)
       || (source->rtyp==LIST_CMD)
-      || ((source->rtyp==IDHDL)&&(IDTYP((idhdl)source->data)==LIST_CMD)))
+      || ((source->rtyp==IDHDL)
+          &&((IDTYP((idhdl)source->data)==LIST_CMD)
+            || (IDTYP((idhdl)source->data)>MAX_TOK)))
+      || (source->rtyp>MAX_TOK))
         return (void *)omStrDup((char *)d);
       else if (e->next==NULL)
       {
@@ -630,10 +716,17 @@ void * sleftv::CopyD(int t)
   if ((rtyp!=IDHDL)&&(rtyp!=ALIAS_CMD)&&(e==NULL))
   {
     if (iiCheckRing(t)) return NULL;
-    void *x=data;
-    if (rtyp==VNOETHER) x=(void *)pCopy(ppNoether);
-    else if ((rtyp==VMINPOLY)&& (currRing->minpoly!=NULL)&&(!rField_is_GF()))
-      x=(void *)nCopy(currRing->minpoly);
+    void *x = data;
+    if (rtyp==VNOETHER) x = (void *)pCopy((currRing->ppNoether));
+    else if ((rtyp==VMINPOLY) && nCoeff_is_algExt(currRing->cf) && (!nCoeff_is_GF(currRing->cf)))
+    {
+      const ring A = currRing->cf->extRing;
+
+      assume( A != NULL );
+      assume( A->qideal != NULL );
+
+      x=(void *)p_Copy(A->qideal->m[0], A);
+    }
     data=NULL;
     return x;
   }
@@ -645,7 +738,7 @@ void * sleftv::CopyD(int t)
 //void * sleftv::CopyD()
 //{
   //if ((rtyp!=IDHDL)&&(e==NULL)
-  //&&(rtyp!=VNOETHER)&&(rtyp!=LIB_CMD)&&(rtyp!=VMINPOLY))
+  //&&(rtyp!=VNOETHER)&&(rtyp!=VMINPOLY))
   //{
   //  void *x=data;
   //  data=NULL;
@@ -722,33 +815,35 @@ char *  sleftv::String(void *d, BOOLEAN typed, int dim)
             char* ps = pString((poly) d);
             s = (char*) omAlloc(strlen(ps) + 10);
             sprintf(s,"%s(%s)", (t /*Typ()*/ == POLY_CMD ? "poly" : "vector"), ps);
-	    omFree(ps);
+            omFree(ps);
             return s;
           }
           else
             return pString((poly)d);
 
+        #ifdef SINGULAR_4_1
+        case CRING_CMD:
+          return nCoeffString((coeffs)d);
+        #endif
+        #ifdef SINGULAR_4_2
+        case CNUMBER_CMD:
+          return n2String((number2)d,typed);
+        case CMATRIX_CMD:
+          {
+            bigintmat *b=(bigintmat*)d;
+            return b->String();
+          }
+        #endif
+
         case NUMBER_CMD:
           StringSetS((char*) (typed ? "number(" : ""));
-          if ((rtyp==IDHDL)&&(IDTYP((idhdl)data)==NUMBER_CMD))
+          if((rtyp==VMINPOLY)&&(rField_is_GF(currRing)))
           {
-            nWrite(IDNUMBER((idhdl)data));
-          }
-          else if (rtyp==NUMBER_CMD)
-          {
-            number n=(number)data;
-            nWrite(n);
-            data=(char *)n;
-          }
-          else if((rtyp==VMINPOLY)&&(rField_is_GF()))
-          {
-            nfShowMipo();
+            nfShowMipo(currRing->cf);
           }
           else
           {
-            number n=nCopy((number)d);
-            nWrite(n);
-            nDelete(&n);
+            nWrite((number)d);
           }
           StringAppendS((char*) (typed ? ")" : ""));
           return StringEndS();
@@ -757,13 +852,13 @@ char *  sleftv::String(void *d, BOOLEAN typed, int dim)
           {
           StringSetS((char*) (typed ? "bigint(" : ""));
           number nl=(number)d;
-          nlWrite(nl,NULL);
+          n_Write(nl,coeffs_BIGINT);
           StringAppendS((char*) (typed ? ")" : ""));
           return StringEndS();
           }
 
         case MATRIX_CMD:
-          s= iiStringMatrix((matrix)d,dim);
+          s= iiStringMatrix((matrix)d,dim, currRing);
           if (typed)
           {
             char* ns = (char*) omAlloc(strlen(s) + 40);
@@ -780,7 +875,7 @@ char *  sleftv::String(void *d, BOOLEAN typed, int dim)
         case MODUL_CMD:
         case IDEAL_CMD:
         case MAP_CMD:
-          s= iiStringMatrix((matrix)d,dim);
+          s= iiStringMatrix((matrix)d,dim, currRing);
           if (typed)
           {
             char* ns = (char*) omAlloc(strlen(s) + 10);
@@ -824,22 +919,23 @@ char *  sleftv::String(void *d, BOOLEAN typed, int dim)
             char* ns = (char*) omAlloc0(strlen(s) + 40);
             sprintf(ns, "bigintmat(bigintvec(%s),%d,%d)", s, bim->rows(), bim->cols());
             omCheckAddr(ns);
-            omFree(s);
             return ns;
           }
           else
-            return s;
-        } 
+            return omStrDup(s);
+        }
+
         case RING_CMD:
-        case QRING_CMD:
           s  = rString((ring)d);
 
           if (typed)
           {
             char* ns;
-            if (t/*Typ()*/ == QRING_CMD)
+            ring r=(ring)d;
+            if (r->qideal!=NULL)
             {
-              char* id = iiStringMatrix((matrix) ((ring) d)->qideal, dim);
+              char* id = iiStringMatrix((matrix) ((ring) d)->qideal, dim,
+                              currRing);
               ns = (char*) omAlloc(strlen(s) + strlen(id) + 20);
               sprintf(ns, "\"%s\";%sideal(%s)", s,(dim == 2 ? "\n" : " "), id);
             }
@@ -904,6 +1000,7 @@ char *  sleftv::String(void *d, BOOLEAN typed, int dim)
   return omStrDup("");
 }
 
+
 int  sleftv::Typ()
 {
   if (e==NULL)
@@ -929,8 +1026,10 @@ int  sleftv::Typ()
       case VSHORTOUT:
         return INT_CMD;
       case VMINPOLY:
+        data=NULL;
         return NUMBER_CMD;
       case VNOETHER:
+        data=NULL;
         return POLY_CMD;
       //case COMMAND:
       //  return COMMAND;
@@ -940,10 +1039,22 @@ int  sleftv::Typ()
   }
   int r=0;
   int t=rtyp;
-  if (t==IDHDL) t=IDTYP((idhdl)data);
-  else if (t==ALIAS_CMD) { idhdl h=(idhdl)IDDATA((idhdl)data); t=IDTYP(h); }
+  void *d=data;
+  if (t==IDHDL) t=IDTYP((idhdl)d);
+  else if (t==ALIAS_CMD)
+  { idhdl h=(idhdl)IDDATA((idhdl)data); t=IDTYP(h);d=IDDATA(h); }
   switch (t)
   {
+#ifdef SINGULAR_4_2
+    case CMATRIX_CMD:
+    {
+      bigintmat *b=(bigintmat*)d;
+      if ((currRing!=NULL)&&(currRing->cf==b->basecoeffs()))
+        return NUMBER_CMD;
+      else
+        return CNUMBER_CMD;
+    }
+#endif
     case INTVEC_CMD:
     case INTMAT_CMD:
       r=INT_CMD;
@@ -972,13 +1083,8 @@ int  sleftv::Typ()
       if ((t==LIST_CMD)||((b!=NULL)&&BB_LIKE_LIST(b)))
       {
         lists l;
-        if (rtyp==IDHDL) l=IDLIST((idhdl)data);
-        else if (rtyp==ALIAS_CMD)
-        {
-          idhdl h=(idhdl)data;
-          l=(lists)(((idhdl)h->data.ustring)->data.ustring);
-        }
-        else             l=(lists)data;
+        if (rtyp==IDHDL) l=IDLIST((idhdl)d);
+        else             l=(lists)d;
         if ((0<e->start)&&(e->start<=l->nr+1))
         {
           Subexpr tmp=l->m[e->start-1].e;
@@ -990,7 +1096,7 @@ int  sleftv::Typ()
         else
         {
           //Warn("out of range: %d not in 1..%d",e->start,l->nr+1);
-          r=NONE;
+          r=DEF_CMD;
         }
       }
       else
@@ -1033,6 +1139,10 @@ int  sleftv::LTyp()
   return Typ();
 }
 
+#ifdef SINGULAR_4_2
+static snumber2 iiNumber2Data[4];
+static int iiCmatrix_index=0;
+#endif
 void * sleftv::Data()
 {
   if ((rtyp!=IDHDL) && iiCheckRing(rtyp))
@@ -1056,17 +1166,23 @@ void * sleftv::Data()
       case VMAXMULT:   return (void *)(long)Kstd1_mu;
       case TRACE:      return (void *)(long)traceit;
       case VSHORTOUT:  return (void *)(long)(currRing != NULL ? currRing->ShortOut : 0);
-      case VMINPOLY:   if (currRing != NULL &&
-                           (currRing->minpoly!=NULL)&&(!rField_is_GF()))
-                       /* Q(a), Fp(a), but not GF(q) */
-                         return (void *)currRing->minpoly;
-                       else
-                         return (void *)nNULL;
-      case VNOETHER:   return (void *) ppNoether;
+      case VMINPOLY:
+        if ( (currRing != NULL)  && nCoeff_is_algExt(currRing->cf) && !nCoeff_is_GF(currRing->cf))
+        {
+          /* Q(a), Fp(a), but not GF(q) */
+          const ring A = currRing->cf->extRing;
+
+          assume( A != NULL );
+          assume( A->qideal != NULL );
+
+          return (void *)A->qideal->m[0];
+        }
+        else
+          return (void *)currRing->cf->nNULL;
+
+      case VNOETHER:   return (void *) (currRing->ppNoether);
       case IDHDL:
         return IDDATA((idhdl)data);
-      case POINTER_CMD:
-        return IDDATA((idhdl)data);
       case COMMAND:
         //return NULL;
       default:
@@ -1099,7 +1215,7 @@ void * sleftv::Data()
       if ((index<1)||(index>iv->length()))
       {
         if (!errorreported)
-          Werror("wrong range[%d] in intvec(%d)",index,iv->length());
+          Werror("wrong range[%d] in intvec %s(%d)",index,this->Name(),iv->length());
       }
       else
         r=(char *)(long)((*iv)[index-1]);
@@ -1114,8 +1230,8 @@ void * sleftv::Data()
          ||(e->next->start>iv->cols()))
       {
         if (!errorreported)
-        Werror("wrong range[%d,%d] in intmat(%dx%d)",index,e->next->start,
-                                                     iv->rows(),iv->cols());
+        Werror("wrong range[%d,%d] in intmat %s(%dx%d)",index,e->next->start,
+                                           this->Name(),iv->rows(),iv->cols());
       }
       else
         r=(char *)(long)(IMATELEM((*iv),index,e->next->start));
@@ -1130,13 +1246,36 @@ void * sleftv::Data()
          ||(e->next->start>m->cols()))
       {
         if (!errorreported)
-        Werror("wrong range[%d,%d] in bigintmat(%dx%d)",index,e->next->start,
-                                                     m->rows(),m->cols());
+        Werror("wrong range[%d,%d] in bigintmat %s(%dx%d)",index,e->next->start,
+                                                     this->Name(),m->rows(),m->cols());
       }
       else
         r=(char *)(BIMATELEM((*m),index,e->next->start));
       break;
     }
+#ifdef SINGULAR_4_2
+    case CMATRIX_CMD:
+    {
+      bigintmat *m=(bigintmat *)d;
+      if ((index<1)
+         ||(index>m->rows())
+         ||(e->next->start<1)
+         ||(e->next->start>m->cols()))
+      {
+        if (!errorreported)
+        Werror("wrong range[%d,%d] in matrix %s(%dx%d)",index,e->next->start,
+                                                     this->Name(),m->rows(),m->cols());
+      }
+      else
+      {
+        iiNumber2Data[iiCmatrix_index].cf=m->basecoeffs();
+        iiNumber2Data[iiCmatrix_index].n=BIMATELEM((*m),index,e->next->start);
+        r=(char*)&iiNumber2Data[iiCmatrix_index];
+        iiCmatrix_index=(iiCmatrix_index+1) % 4;
+      }
+      break;
+    }
+#endif
     case IDEAL_CMD:
     case MODUL_CMD:
     case MAP_CMD:
@@ -1145,7 +1284,7 @@ void * sleftv::Data()
       if ((index<1)||(index>IDELEMS(I)))
       {
         if (!errorreported)
-          Werror("wrong range[%d] in ideal/module(%d)",index,IDELEMS(I));
+          Werror("wrong range[%d] in ideal/module %s(%d)",index,this->Name(),IDELEMS(I));
       }
       else
         r=(char *)I->m[index-1];
@@ -1201,8 +1340,9 @@ void * sleftv::Data()
          ||(e->next->start>MATCOLS((matrix)d)))
       {
         if (!errorreported)
-          Werror("wrong range[%d,%d] in intmat(%dx%d)",
+          Werror("wrong range[%d,%d] in matrix %s(%dx%d)",
                   index,e->next->start,
+                  this->Name(),
                   MATROWS((matrix)d),MATCOLS((matrix)d));
       }
       else
@@ -1248,10 +1388,10 @@ void * sleftv::Data()
           }
         }
         else //if (!errorreported)
-          Werror("wrong range[%d] in list(%d)",index,l->nr+1);
+          Werror("wrong range[%d] in list %s(%d)",index,this->Name(),l->nr+1);
       }
       else
-        Werror("cannot index type %s(%d)",Tok2Cmdname(t),t);
+        Werror("cannot index %s of type %s(%d)",this->Name(),Tok2Cmdname(t),t);
       break;
     }
   }
@@ -1342,9 +1482,10 @@ leftv sleftv::LHdl()
 
 BOOLEAN assumeStdFlag(leftv h)
 {
-  if ((h->e!=NULL)&&(h->LTyp()==LIST_CMD))
+  if (h->e!=NULL)
   {
-    return assumeStdFlag(h->LData());
+    leftv hh=h->LData();
+    if (h!=hh) return assumeStdFlag(h->LData());
   }
   if (!hasFlag(h,FLAG_STD))
   {
@@ -1365,7 +1506,7 @@ BOOLEAN assumeStdFlag(leftv h)
 * into an expression (sleftv), deletes the string
 * utility for grammar and iparith
 */
-void syMake(leftv v,const char * id, idhdl packhdl)
+void syMake(leftv v,const char * id, package pa)
 {
   /* resolv an identifier: (to DEF_CMD, if siq>0)
   * 1) reserved id: done by scanner
@@ -1374,7 +1515,7 @@ void syMake(leftv v,const char * id, idhdl packhdl)
   * 4) ringvar, ringpar, local ring
   * 5) existing identifier, global
   * 6) monom (resp. number), local ring: consisting of:
-  * 6') ringvar, ringpar, global ring
+  * 6') ringvar,  ringpar,global ring
   * 6'') monom (resp. number), local ring
   * 7) monom (resp. number), non-local ring
   * 8) basering
@@ -1389,10 +1530,9 @@ void syMake(leftv v,const char * id, idhdl packhdl)
 #endif
   idhdl save_ring=currRingHdl;
   v->Init();
-  if(packhdl != NULL)
+  if(pa != NULL)
   {
-  //  Print("setting req_packhdl to %s\n",IDID(packhdl));
-    v->req_packhdl = IDPACKAGE(packhdl);
+    v->req_packhdl = pa;
   }
   else v->req_packhdl = currPack;
 //  if (v->req_packhdl!=basePack)
@@ -1454,7 +1594,7 @@ void syMake(leftv v,const char * id, idhdl packhdl)
     /*&& (!yyInRingConstruction)*/)
     {
       int vnr;
-      if ((vnr=rIsRingVar(id))>=0)
+      if ((vnr=r_IsRingVar(id, currRing->names,currRing->N))>=0)
       {
         poly p=pOne();
         pSetExp(p,vnr+1,1);
@@ -1464,26 +1604,27 @@ void syMake(leftv v,const char * id, idhdl packhdl)
         v->rtyp = POLY_CMD;
         return;
       }
-      if ((currRing->algring!=NULL) && ((vnr=r_IsRingVar(id,currRing->algring))>=0))
+      if((n_NumberOfParameters(currRing->cf)>0)
+      &&((vnr=r_IsRingVar(id, (char**)n_ParameterNames(currRing->cf),
+                              n_NumberOfParameters(currRing->cf))>=0)))
       {
-	BOOLEAN ok=FALSE;
-	poly p = pmInit(id,ok);
-	if (ok && (p!=NULL))
-	{
-          v->data = (void *)pGetCoeff(p);
-	  v->rtyp = NUMBER_CMD;
-          v->name = id;
+        BOOLEAN ok=FALSE;
+        poly p = pmInit(id,ok);
+        if (ok && (p!=NULL))
+        {
+          v->data = pGetCoeff(p);
           pGetCoeff(p)=NULL;
           pLmFree(p);
+          v->rtyp = NUMBER_CMD;
+          v->name = id;
           return;
-	}
+        }
       }
-
     }
     /* 5. existing identifier, global */
     if (h!=NULL)
     {
-      if (id!=IDID(h)) omFreeBinAddr((ADDRESS)id); /*assume strlen(id) <1000 */
+      if (id!=IDID(h)) omFreeBinAddr((ADDRESS)id);  /*assume strlen(id) <1000 */
       goto id_found;
     }
     /* 6. local ring: number/poly */
@@ -1502,7 +1643,7 @@ void syMake(leftv v,const char * id, idhdl packhdl)
           // in this case we may have monomials equal to 0 in p_Read
           v->name = id;
           #else
-          omFreeBinAddr((ADDRESS)id); /*assume strlen(id) <1000 */
+          omFreeBinAddr((ADDRESS)id);
           #endif
         }
         else if (pIsConstant(p))
@@ -1519,14 +1660,6 @@ void syMake(leftv v,const char * id, idhdl packhdl)
           v->rtyp = POLY_CMD;
           v->name = id;
         }
-        if (TEST_V_ALLWARN /*&& (myynest>0)*/
-        && ((rIsRingVar(id)>=0)
-          || ((currRing->P>0)
-             &&(r_IsRingVar(id, currRing->algring)>=0))))
-        {
-        // WARNING: do not use ring variable names in procedures
-          Warn("use of variable >>%s<< in a procedure in line %s",id,my_yylinebuf);
-        }
         return;
       }
     }
@@ -1548,7 +1681,7 @@ void syMake(leftv v,const char * id, idhdl packhdl)
           // in this case we may have monomials equal to 0 in p_Read
           v->name = id;
           #else
-          omFreeBinAddr((ADDRESS)id); /*assume strlen(id) <1000 */
+          omFreeBinAddr((ADDRESS)id);
           #endif
         }
         else
@@ -1566,6 +1699,15 @@ void syMake(leftv v,const char * id, idhdl packhdl)
           v->rtyp = POLY_CMD;
           v->name = id;
         }
+        //if (TEST_V_ALLWARN /*&& (myynest>0)*/
+        //&& ((r_IsRingVar(id, currRing->names,currRing->N)>=0)
+        //  || ((n_NumberOfParameters(currRing->cf)>0)
+        //     &&(r_IsRingVar(id, (char**)n_ParameterNames(currRing->cf),
+        //                        n_NumberOfParameters(currRing->cf))>=0))))
+        //{
+        //// WARNING: do not use ring variable names in procedures
+        //  Warn("use of variable >>%s<< in a procedure in line %s",id,my_yylinebuf);
+        //}
         return;
       }
     }
@@ -1574,7 +1716,7 @@ void syMake(leftv v,const char * id, idhdl packhdl)
     {
       if (strcmp(id,IDID(currRingHdl))==0)
       {
-        if (IDID(currRingHdl)!=id) omFreeBinAddr((ADDRESS)id); /*assume strlen(id) <1000 */
+        if (IDID(currRingHdl)!=id) omFreeBinAddr((ADDRESS)id); /*assume strlen (id) <1000 */
         h=currRingHdl;
         goto id_found;
       }
@@ -1654,8 +1796,12 @@ int sleftv::Eval()
         if(!nok)
         {
           nok=iiMake_proc(h,req_packhdl,&d->arg2);
+          this->CleanUp(currRing);
           if (!nok)
+          {
             memcpy(this,&iiRETURNEXPR,sizeof(sleftv));
+            memset(&iiRETURNEXPR,0,sizeof(sleftv));
+          }
         }
       }
       else nok=TRUE;
@@ -1709,6 +1855,7 @@ int sleftv::Eval()
     }
     else
     {
+      sleftv tmp; tmp.Init();
       int toktype=iiTokType(d->op);
       if ((toktype==CMD_M)
       ||( toktype==ROOT_DECL_LIST)
@@ -1732,51 +1879,51 @@ int sleftv::Eval()
             d->arg3.Init();
           }
           if (d->argc==0)
-            nok=nok||iiExprArithM(this,NULL,d->op);
+            nok=nok||iiExprArithM(&tmp,NULL,d->op);
           else
-            nok=nok||iiExprArithM(this,&d->arg1,d->op);
+            nok=nok||iiExprArithM(&tmp,&d->arg1,d->op);
         }
         else
         {
           nok=d->arg1.Eval();
-          nok=nok||iiExprArithM(this,&d->arg1,d->op);
+          nok=nok||iiExprArithM(&tmp,&d->arg1,d->op);
         }
       }
       else if (d->argc==1)
       {
         nok=d->arg1.Eval();
-        nok=nok||iiExprArith1(this,&d->arg1,d->op);
+        nok=nok||iiExprArith1(&tmp,&d->arg1,d->op);
       }
       else if(d->argc==2)
       {
         nok=d->arg1.Eval();
         nok=nok||d->arg2.Eval();
-        nok=nok||iiExprArith2(this,&d->arg1,d->op,&d->arg2);
+        nok=nok||iiExprArith2(&tmp,&d->arg1,d->op,&d->arg2);
       }
       else if(d->argc==3)
       {
         nok=d->arg1.Eval();
         nok=nok||d->arg2.Eval();
         nok=nok||d->arg3.Eval();
-        nok=nok||iiExprArith3(this,d->op,&d->arg1,&d->arg2,&d->arg3);
+        nok=nok||iiExprArith3(&tmp,d->op,&d->arg1,&d->arg2,&d->arg3);
       }
       else if(d->argc!=0)
       {
         nok=d->arg1.Eval();
-        nok=nok||iiExprArithM(this,&d->arg1,d->op);
+        nok=nok||iiExprArithM(&tmp,&d->arg1,d->op);
       }
       else // d->argc == 0
       {
-        nok = iiExprArithM(this, NULL, d->op);
+        nok = iiExprArithM(&tmp, NULL, d->op);
       }
+      this->CleanUp();
+      memcpy(this,&tmp,sizeof(tmp));
     }
   }
   else if (((rtyp==0)||(rtyp==DEF_CMD))
     &&(name!=NULL))
   {
      syMake(this,name);
-     //if (rtyp==0) /*not found */
-     //  nok=TRUE;
   }
 #ifdef MDEBUG
   switch(Typ())
@@ -1788,7 +1935,7 @@ int sleftv::Eval()
       break;
     case BIGINT_CMD:
 #ifdef LDEBUG
-      nlTest((number)Data());
+      n_Test((number)Data(),coeffs_BIGINT);
 #endif
       break;
     case POLY_CMD:
